# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys
import gtk
import time
import gobject
import gettext

if __name__ == '__main__':
    from emesenecommon import PATH

    if os.path.exists("default.mo"):
        gettext.GNUTranslations(open("default.mo")).install()
    elif os.path.exists(PATH.LANG_PATH):
        gettext.install('emesene', PATH.LANG_PATH)
    else:
        gettext.install('emesene')

import htmltextview
import plugins_base.Logger as Logger

class LogViewer(gtk.Window):
    '''the main window of the log viewer'''

    def __init__(self, exit_on_quit=False):
        '''class constructor'''
        gtk.Window.__init__(self)

        self.set_default_size(800, 600)
        self.set_title(_('Log viewer'))

        self.logger = None

        self.template = {}
        self.template['nick-changed'] = \
          _('[%(date)s] %(mail)s changed the nick to %(data)s\n')
        self.template['personal-message-changed'] = \
         _('[%(date)s] %(mail)s changed the personal message to %(data)s\n')
        self.template['status-changed'] = \
         _('[%(date)s] %(mail)s changed his status to %(data)s\n')
        self.template['user-offline'] = \
         _('[%(date)s] %(mail)s is now offline\n')
        self.template['message'] = \
         _('[%(date)s] %(mail)s: %(data)s\n')
        self.template['nudge'] = \
         _('[%(date)s] %(mail)s just sent you a nudge\n')
        self.template['action-message'] = \
         _('[%(date)s] %(mail)s %(data)s\n')

        self.template_default = \
          _('[%(date)s] %(mail)s %(event)s %(data)s\n')

        self.exit_on_quit = exit_on_quit

        self.main_vbox = gtk.VBox()
        self.main_vbox.set_spacing(2)
        self.menu = LogViewerMenu()
        self.paned = gtk.HPaned()
        self.left_panel = LeftPanel()
        self.right_panel = RightPanel()
        self.status = gtk.Statusbar()

        self.paned.add1(self.left_panel)
        self.paned.add2(self.right_panel)
        
        self.left_panel.mail_filter.connect('filter-changed',
            self.on_mail_filter_changed)
        self.left_panel.mail_filter.connect('all-selected',
            self.on_mail_filter_all_selected)
        self.left_panel.mail_filter.connect('all-deselected',
            self.on_mail_filter_all_deselected)

        self.left_panel.user_event_filter.connect('filter-changed',
            self.on_user_event_filter_changed)
        self.left_panel.user_event_filter.connect('all-selected',
            self.on_user_event_filter_all_selected)
        self.left_panel.user_event_filter.connect('all-deselected',
            self.on_user_event_filter_all_deselected)

        self.left_panel.conv_event_filter.connect('filter-changed',
            self.on_user_event_filter_changed)
        self.left_panel.conv_event_filter.connect('all-selected',
            self.on_user_event_filter_all_selected)
        self.left_panel.conv_event_filter.connect('all-deselected',
            self.on_user_event_filter_all_deselected)

        self.menu.connect('open-file', self.on_open_file)
        self.menu.connect('clear', self.on_clear)
        self.menu.connect('quit', self.on_quit)
        self.left_panel.b_run.connect('clicked', self.on_b_run_clicked)

        self.connect('delete-event', self.on_quit)

        self.main_vbox.pack_start(self.menu, False)
        self.main_vbox.pack_start(self.paned)
        self.main_vbox.pack_start(self.status, False)

        self.main_vbox.show_all()
        self.add(self.main_vbox)

    def open_file(self, filename):
        '''open the db and do all the things needed to show the info'''
        self.logger = Logger.Logger(filename)
        self.left_panel.mail_filter.clear()
        self.left_panel.user_event_filter.clear()
        self.left_panel.conv_event_filter.clear()

        for mail in self.logger.get_mails():
            self.left_panel.mail_filter.append((False, mail))
        
        for event in self.logger.get_user_events():
            self.left_panel.user_event_filter.append((False, event))
        
        for event in self.logger.get_conversation_events():
            self.left_panel.conv_event_filter.append((False, event))

    def on_open_file(self, widget):
        '''called when the open option is clicked, show a open dialog'''
        dialog = gtk.FileChooserDialog(_('Open a log file'), 
                    self, gtk.FILE_CHOOSER_ACTION_OPEN, 
                    (_( 'Open'), gtk.RESPONSE_OK, 
                    _('Cancel'), gtk.RESPONSE_CANCEL))

        #dialog.set_select_multiple(True)

        filter = gtk.FileFilter()
        filter.add_pattern('*.db')
        filter.set_name('*.db')
        dialog.add_filter(filter)

        response = dialog.run()
        dialog.hide()

        if response == gtk.RESPONSE_OK:
            #for file in dialog.get_filenames():
            self.open_file(dialog.get_filename())
    
    def on_clear(self, widget):
        '''clear the content of the results'''
        self.left_panel.mail_filter.clear()
        self.left_panel.user_event_filter.clear()
        self.left_panel.conv_event_filter.clear()
        self.right_panel.clear()
        self.logger = None

    def on_quit(self, *args):
        '''quit the program'''
        if self.exit_on_quit:
            sys.exit(0)
        else:
            self.hide()

    def on_mail_filter_changed(self, filter_list, state, mail):
        '''called when a value changes on the mail filter'''

        print state, mail

    def on_mail_filter_all_selected(self, filter_list):
        '''called when all values are selected on the mail filter'''

        print 'all mails selected'

    def on_mail_filter_all_deselected(self, filter_list):
        '''called when all values are desellected on the mail filter'''

        print 'all mails deselected'
    
    def on_user_event_filter_changed(self, filter_list, state, event):
        '''called when a value changes on the event filter'''

        print state, event

    def on_user_event_filter_all_selected(self, filter_list):
        '''called when all values are selected on the event filter'''

        print 'all events selected'

    def on_user_event_filter_all_deselected(self, filter_list):
        '''called when all values are deselected on the event filter'''

        print 'all events deselected'

    def on_conv_event_filter_changed(self, filter_list, state, event):
        '''called when a value changes on the event filter'''

        print state, event

    def on_conv_event_filter_all_selected(self, filter_list):
        '''called when all values are selected on the event filter'''

        print 'all events selected'

    def on_conv_event_filter_all_deselected(self, filter_list):
        '''called when all values are deselected on the event filter'''

        print 'all events deselected'

    def on_b_run_clicked(self, widget):
        '''called when the run button is clicked'''
        self.show_user_events()
        self.show_conv_events()

    def show_user_events(self):
        '''show the data that match with the user selection'''
        mails = self.left_panel.mail_filter.get_by_status(True)
        user_events = self.left_panel.user_event_filter.get_by_status(True)

        self.right_panel.set_user_text('')

        mail_list = '"' + '","'.join(mails) + '"'
        event_list = '"' + '","'.join(user_events) + '"'

        t1 = time.time()
        for (stamp, name, account, data) in self.logger.query('\
        select e.stamp, e.name, u.account, ue.data \
        from event e, user_event ue, user u \
        where ue.id_event = e.id and \
        ue.id_user = u.id and \
        u.account in (%s) and \
        e.name in (%s) \
        order by e.stamp, u.account, e.name;' % (mail_list, event_list)):
            self.right_panel.append_user_text(self.format_user_event(stamp, 
                name, account, data))

        t2 = time.time()
        print 'time user: ', t2 - t1
    
    def show_conv_events(self):
        '''show the data that match with the user selection'''
        mails = self.left_panel.mail_filter.get_by_status(True)
        conv_events = self.left_panel.conv_event_filter.get_by_status(True)

        self.right_panel.set_conv_text('')

        mail_list = '"' + '","'.join(mails) + '"'
        event_list = '"' + '","'.join(conv_events) + '"'

        t1 = time.time()
        for (stamp, name, account, data) in self.logger.query('\
        select e.stamp, e.name, u.account, ue.data \
        from event e, conversation_event ue, user u \
        where ue.id_event = e.id and \
        ue.id_user = u.id and \
        u.account in (%s) and \
        e.name in (%s) \
        order by e.stamp, u.account, e.name;' % (mail_list, event_list)):
            self.right_panel.append_conv_text(self.format_conv_event(stamp, 
                name, account, data))

        t2 = time.time()
        print 'time conversation: ', t2 - t1

    def format_user_event(self, stamp, name, account, data):
        '''give a specific format to known events'''
        if name in self.template:
            return self.template[name] % self.map_user_event(stamp, name, 
                account, data)
        
        return self.template_default % self.map_user_event(stamp, name,
            account, data)
    
    def format_conv_event(self, stamp, name, account, data):
        '''give a specific format to known events'''
        if name == 'message':
            try:
                data = data.split('\r\n')[2]
            except IndexError:
                print 'invalid message format: %s' % data

        if name in self.template:
            return self.template[name] % self.map_user_event(stamp, name, 
                account, data)
        
        return self.template_default % self.map_user_event(stamp, name,
            account, data)
            
    def map_user_event(self, stamp, name, account, data):
        '''create a dict with the data from a user event that can 
        be used to replace values on the template'''

        return {'mail' : account, 
        'data' : gobject.markup_escape_text(data), 
        'date' : gobject.markup_escape_text(time.ctime(stamp)), 
        'event' : gobject.markup_escape_text(name)}

class LogViewerMenu(gtk.MenuBar):
    '''the menu from the main window'''

    __gsignals__ = {
        'open-file' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        'clear' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        'quit' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()) }

    def __init__(self):
        gtk.MenuBar.__init__(self)

        # build the file menu

        file_menu = gtk.Menu()
        file_menu_item = gtk.ImageMenuItem(_('_File'))
        file_menu_item.set_submenu(file_menu)

        open_menu_item = gtk.ImageMenuItem(gtk.STOCK_OPEN)
        file_menu.add(open_menu_item)
        clear_menu_item = gtk.ImageMenuItem(gtk.STOCK_CLEAR)
        file_menu.add(clear_menu_item)
        file_menu.add(gtk.SeparatorMenuItem())
        quit_menu_item = gtk.ImageMenuItem(gtk.STOCK_QUIT)
        file_menu.add(quit_menu_item)

        open_menu_item.connect("activate" , self.on_open_activate)
        clear_menu_item.connect("activate" , self.on_clear_activate)
        quit_menu_item.connect("activate" , self.on_quit_activate)

        self.add(file_menu_item)
        file_menu.show_all()

    def on_open_activate(self, widget):
        self.emit('open-file')
    
    def on_clear_activate(self, widget):
        self.emit('clear')

    def on_quit_activate(self, widget):
        self.emit('quit')

class LeftPanel(gtk.VBox):
    '''the left panel of the main window'''

    def __init__(self):
        '''class constructor'''
        gtk.VBox.__init__(self)
        self.set_border_width(2)
        self.set_spacing(2)
        
        self.tabs = gtk.Notebook()
        self.calendar = gtk.Calendar()
        self.mail_filter = FilterList(_('Contact'))
        self.user_event_filter = FilterList(_('Name'))
        self.conv_event_filter = FilterList(_('Name'))
        self.b_run = gtk.Button(stock=gtk.STOCK_EXECUTE)

        self.tabs.append_page(self.mail_filter, gtk.Label(_('Contacts')))
        self.tabs.append_page(self.user_event_filter,gtk.Label(
            _('User Events')))
        self.tabs.append_page(self.conv_event_filter,gtk.Label(
            _('Conversation Events')))

        self.pack_start(self.tabs, True, True)
        #self.pack_start(self.calendar, False)
        self.pack_start(self.b_run, False)

        self.show_all() 

class RightPanel(gtk.VBox):

    '''the right panel of the main window'''

    def __init__(self):
        '''class constructor'''
        gtk.VBox.__init__(self)
        self.set_border_width(2)

        self.user_view = gtk.TextView()#htmltextview.HtmlTextView()
        self.user_buffer = self.user_view.get_buffer()
        self.user_scroll = gtk.ScrolledWindow()
        self.user_scroll.add(self.user_view)
        self.user_scroll.set_policy(gtk.POLICY_AUTOMATIC, 
            gtk.POLICY_AUTOMATIC)
        
        self.conv_view = gtk.TextView()#htmltextview.HtmlTextView()
        self.conv_buffer = self.conv_view.get_buffer()
        self.conv_scroll = gtk.ScrolledWindow()
        self.conv_scroll.add(self.conv_view)
        self.conv_scroll.set_policy(gtk.POLICY_AUTOMATIC, 
            gtk.POLICY_AUTOMATIC)

        notebook = gtk.Notebook()
        notebook.append_page(self.user_scroll, gtk.Label(_('User events')))
        notebook.append_page(self.conv_scroll, 
            gtk.Label(_('Conversation events')))

        self.pack_start(notebook, True, True)

        self.show_all()

    def clear(self):
        '''clear the content of the view'''
        #textbuff = self.view.get_buffer()
        #start, end = textbuff.get_bounds()
        #textbuff.delete(start,end)
        self.user_buffer.set_text('')
        self.conv_buffer.set_text('')

    def set_user_text(self, text):
        '''set the text of the view'''
        self.user_buffer.set_text(text)
    
    def set_conv_text(self, text):
        '''set the text of the view'''
        self.conv_buffer.set_text(text)

    def append_user_text(self, text):
        '''append text to the view'''
        self.user_buffer.insert(self.user_buffer.get_end_iter(), text)
    
    def append_conv_text(self, text):
        '''append text to the view'''
        self.conv_buffer.insert(self.conv_buffer.get_end_iter(), text)

class FilterList(gtk.VBox):
    '''a class that represent a list of items with checkboxes and 
    2 buttons, select all, deselect all'''

    __gsignals__ = {
        'filter-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
        'all-selected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
        'all-deselected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
    }

    def __init__(self, column_name):
        '''class constructor'''
        gtk.VBox.__init__(self)
        self.set_border_width(2)

        self.scroll = gtk.ScrolledWindow()
        self.model = gtk.ListStore(bool, str)
        self.list = gtk.TreeView(self.model)

        cr_toggle = gtk.CellRendererToggle()
        cr_text = gtk.CellRendererText()

        col_toggle = gtk.TreeViewColumn(_('State'), cr_toggle, active=0)
        col_text = gtk.TreeViewColumn(column_name, cr_text, text=1)

        cr_toggle.connect('toggled', self.active_toggled)

        self.list.append_column(col_toggle)
        self.list.append_column(col_text)

        self.scroll.add(self.list)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        self.b_select_all = gtk.Button(_('Select all'))
        self.b_deselect_all = gtk.Button(_('Deselect all'))

        self.b_select_all.connect('clicked', self.on_b_select_all)
        self.b_deselect_all.connect('clicked', self.on_b_deselect_all)

        hbox = gtk.HBox()
        hbox.set_border_width(2)

        hbox.pack_start(self.b_select_all, True, False)
        hbox.pack_start(self.b_deselect_all, True, False)

        self.pack_start(self.scroll, True, True)
        self.pack_start(hbox, False)

        self.show_all()

    def active_toggled(self, cell, path):
        '''called when a row is toggled'''

        iterator = self.model.get_iter_from_string(path)
        value_bool = self.model.get_value(iterator, 0)
        value_str = self.model.get_value(iterator, 1)
        self.model.set_value(iterator, 0, not value_bool)
        
        self.emit('filter-changed', not value_bool, value_str)

    def get_all_items(self):
        '''return a list of tuples with the values'''
        item_list = []

        for row in self.model:
            item_list.append((row[0], row[1]))

        return item_list

    def get_by_status(self, status):
        '''return a list of tuples with the values that have the status
        "status"'''
        item_list = []

        for row in self.model:
            if row[0] == status:
                item_list.append(row[1])

        return item_list

    def set_all(self, state):
        '''set all the checkboxes to state (True or False)'''

        for row in self.model:
            row[0] = state

    def clear(self):
        '''clear the items on the filter list'''
        self.model.clear()

    def append(self, row):
        '''append a row to the model'''
        self.model.append(row)

    def on_b_select_all(self, widget):
        '''called when the select all is clicked'''
        self.set_all(True)
        self.emit('all-selected')
    
    def on_b_deselect_all(self, widget):
        '''called when the deselect all is clicked'''
        self.set_all(False)
        self.emit('all-deselected')
        
gobject.type_register(LogViewerMenu)

if __name__ == '__main__':
    def test():
        log_viewer = LogViewer(True)
        log_viewer.show()
        gtk.main()

    test()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import time
import gobject
import pango

import Plugin
import stock
import dialog
import desktop
import webbrowser
from emesenelib.common import escape
from emesenelib.common import unescape
from emesenecommon import PATH
from emesenecommon import RESPONSE_NO_AVATAR
from Theme import resizePixbuf

growFactor = 20 # the number of pixels to grow every iteration

class PixmapDialog(gtk.Window):
    '''a dialog to set Notification Pixmap'''
    def __init__(self, response, filename, font, color, online, offline, \
                newMsg, typing, newMail, started, idle, position, scroll):

        gtk.Window.__init__(self)
        self.set_title(_('Notification Config'))
        self.set_border_width(4)
        self.set_position(gtk.WIN_POS_CENTER)

        self.response_ok = response

        self.filename = filename
        self.fontdesc = font
        self.FColor = color

        self.hbox = gtk.HBox()
        self.lbox = gtk.HBox()

        self.image = gtk.Image()

        self.sample = gtk.Label()
        self.sample.set_label('<span foreground="%s">%s</span>' % \
            (self.FColor, _('Sample Text')))
        self.sample.set_use_markup(True)

        try:
            self.sample.modify_font(pango.FontDescription(self.fontdesc))
        except:
            print 'Font Error'

        self.fonttype = gtk.ToolButton()
        self.fonttype.set_stock_id(gtk.STOCK_SELECT_FONT)

        self.fontColor = gtk.ToolButton()
        self.fontColor.set_stock_id(gtk.STOCK_SELECT_COLOR)

        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file(self.filename)
            pixbuf = resizePixbuf(pixbuf, 24,24)
            self.image.set_from_pixbuf(pixbuf)
        except:
            self.filename = None

        self.button = gtk.Button(_('Image'))

        self.hbox.pack_start(self.image)
        self.hbox.pack_start(self.button)

        self.lbox.pack_start(self.fonttype)
        self.lbox.pack_start(self.fontColor)

        self.vbox = gtk.VBox(spacing=4)

        self.vbox.pack_start(self.hbox)
        self.vbox.pack_start(self.sample)
        self.vbox.pack_start(self.lbox)

        self.button.connect('clicked', self.clickPixmap)
        self.fonttype.connect('clicked', self.clickFont)
        self.fontColor.connect('clicked', self.clickColor)

        self.chbuOnline = gtk.CheckButton(_('Notify when someone gets online'))
        self.chbuOnline.set_active(online)
        self.chbuOffline = gtk.CheckButton(_('Notify when someone gets offline'))
        self.chbuOffline.set_active(offline)
        self.chbuNewMail = gtk.CheckButton(_('Notify when receiving an email'))
        self.chbuNewMail.set_active(newMail)
        self.chbuTyping = gtk.CheckButton(_('Notify when someone starts typing'))
        self.chbuTyping.set_active(typing)
        self.chbuNewMsg = gtk.CheckButton(_('Notify when receiving a message'))
        self.chbuNewMsg.set_active(newMsg)
        self.chbuStarted = gtk.CheckButton(_('Don`t notify if conversation is started'))
        self.chbuStarted.set_active(started)
        self.chbuIdle = gtk.CheckButton(_('Disable notifications when busy'))
        self.chbuIdle.set_active(idle)

        self.lblPos = gtk.Label()
        self.lblPos.set_label(_('Position'))
        self.coboPosition = gtk.combo_box_new_text()
        self.coboPosition.append_text(_('Top Left'))
        self.coboPosition.append_text(_('Top Right'))
        self.coboPosition.append_text(_('Bottom Left'))
        self.coboPosition.append_text(_('Bottom Right'))
        self.coboPosition.set_active(position)

        self.pbox = gtk.HBox()
        self.pbox.pack_start(self.lblPos)
        self.pbox.pack_start(self.coboPosition)

        self.lblScr = gtk.Label()
        self.lblScr.set_label(_('Scroll'))
        self.coboScroll = gtk.combo_box_new_text()
        self.coboScroll.append_text(_('Horizontal'))
        self.coboScroll.append_text(_('Vertical'))
        self.coboScroll.set_active(scroll)

        self.sbox = gtk.HBox()
        self.sbox.pack_start(self.lblScr)
        self.sbox.pack_start(self.coboScroll)

        self.vbox.pack_start(self.chbuOnline)
        self.vbox.pack_start(self.chbuOffline)
        self.vbox.pack_start(self.chbuNewMail)
        self.vbox.pack_start(self.chbuTyping)
        self.vbox.pack_start(self.chbuNewMsg)
        self.vbox.pack_start(self.chbuStarted)
        self.vbox.pack_start(self.chbuIdle)
        self.vbox.pack_start(self.pbox)
        self.vbox.pack_start(self.sbox)

        b_accept = gtk.Button(stock=gtk.STOCK_OK)
        b_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)

        b_accept.connect('clicked', self._on_response_ok)
        b_cancel.connect('clicked', self._on_response_cancel)

        hbbox = gtk.HButtonBox()
        hbbox.set_spacing(4)
        hbbox.set_layout(gtk.BUTTONBOX_END)

        hbbox.pack_start(b_cancel, False)
        hbbox.pack_start(b_accept, False)

        self.vbox.pack_start(hbbox)

        self.add(self.vbox)

        self.show_all()

    def clickPixmap(self, arg):
        def _on_image_selected(response, path):
            if response == stock.ACCEPT:
                self.filename = path
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file(self.filename)
                pixbuf = resizePixbuf(pixbuf, 24, 24)
                self.image.set_from_pixbuf(pixbuf)
                self.image.show()
            except:
                self.filename = None
                self.image.hide() 

        dialog.ImageChooser(os.path.expanduser('~'), _on_image_selected).show()

    def clickFont(self, arg):
        fontDialog = gtk.FontSelectionDialog(_('Choose a font'))
        if self.fontdesc != None:
            fontDialog.set_font_name(self.fontdesc)
        response = fontDialog.run()
        if response == gtk.RESPONSE_OK:
            pangoDesc = pango.FontDescription(fontDialog.get_font_name())
            self.sample.modify_font(pangoDesc)
            self.fontdesc = pangoDesc.to_string()
        fontDialog.destroy()

    def clickColor(self, arg):
        colorDialog = gtk.ColorSelectionDialog(_('Choose a color'))
        colorDialog.colorsel.set_has_palette(True)
        response = colorDialog.run()
        if response == gtk.RESPONSE_OK:
            color = colorDialog.colorsel.get_current_color()
            red = color.red >> 8
            green = color.green >> 8
            blue = color.blue >> 8
            self.FColor = '#%02X%02X%02X' % (red, green, blue)
            self.sample.set_label('<span foreground="%s">%s</span>' % \
                    (self.FColor, _('Sample Text')))
            self.sample.set_use_markup(True)
        colorDialog.destroy()

    def _on_response_ok(self, button):
         self.destroy()
         self.response_ok(self.filename, self.fontdesc, self.FColor, \
                 int(self.chbuOnline.get_active()), \
                 int(self.chbuOffline.get_active()), \
                 int(self.chbuNewMsg.get_active()), \
                 int(self.chbuTyping.get_active()), \
                 int(self.chbuNewMail.get_active()), \
                 int(self.chbuStarted.get_active()), \
                 int(self.chbuIdle.get_active()), \
                 self.coboPosition.get_active(), \
                 self.coboScroll.get_active())

    def _on_response_cancel(self, button):
         self.destroy()

class NotificationManager:
    ''' This class manages the creation display and destruction of the notifications. '''

    def __init__(self, defaultHeight = 128, defaultWidth = 200):
        ''' Contructor '''

        self.defaultHeight = defaultHeight
        self.defaultWidth = defaultWidth

        self.offset = 0

        #[[Notification, timestamp], [Notification, timestamp]]
        self.list = []

        self.animate = None

    def newNotification(self, string, pos, scroll, pixmap = None, \
                closePixmap = None, callback = None, params = None, \
                userPixbuf = None, font = None, color = None):
        '''
        create a new notification, pixmap is the background image (as a pixbuf),
        closepixmap is a pixbuf for the close button.
        callback is the method that will be called when the message in the Notification
        is clicked
        '''
        if pixmap != None:
            width, height = pixmap.get_size()
        else:
            width = self.defaultWidth
            height = self.defaultHeight

        rgb = gtk.gdk.screen_get_default().get_rgb_colormap()
        gtk.widget_push_colormap(rgb)
        g = Notification(pos, scroll, self.offset, string, height, width, \
                pixmap, closePixmap, callback, params, userPixbuf, font, color)
        g.show()
        gtk.widget_pop_colormap()

        self.offset = g.getOffset()

        self.list.append([g, int(time.time())])

        if len(self.list) <= 1:
            self.animate = gobject.timeout_add(100, self.refresh)

    def refresh(self):
        '''
        check which notifications should be closed
        resize and move notifications
        '''

        self.offset = 0

        if self.list == []:
            return False
        else:
            timestamp = int(time.time())
            count = 0
            for i in self.list:

                if not i[0].get_property('visible'):
                    del self.list[count]
                elif i[1] + 7 <= timestamp:
                    i[0].hide()
                    del self.list[count]
                else:
                    self.list[count][0].grow(self.offset)
                    self.offset = self.list[count][0].getOffset()

                count += 1

            return True

    def closeAll(self):
        ''' close all the notifications '''

        if self.animate:
            gobject.source_remove(self.animate)

        for i in range(len(self.list)):
            self.list[i][0].hide()

        self.offset = 0
        self.list = []

class Notification(gtk.Window):

    def __init__(self, corner, scroll, offset, string, height = 128, \
                width = 200, pixmap = None, closePixmap = None, \
                callback = None, params = None, userPixbuf = None, \
                font = None, color = None):

        gtk.Window.__init__(self)

        if corner == 0:
            self.set_gravity(gtk.gdk.GRAVITY_NORTH_WEST)
        elif corner == 1:
            self.set_gravity(gtk.gdk.GRAVITY_NORTH_EAST)
        elif corner == 2:
            self.set_gravity(gtk.gdk.GRAVITY_SOUTH_WEST)
        else:
            self.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)

        self.set_property('can-focus', False)
        self.set_property('accept-focus', False)

        self.corner = corner
        self.scroll = scroll

        if scroll == 0:
            self.height = height
            self.max = width
            self.width = 1
        else:
            self.width = width
            self.max = height
            self.height = 1

        self.callback = callback

        self.set_geometry_hints(None, min_width=-1, min_height=-1, \
                max_width=width, max_height=height)
        self.set_accept_focus(False)

        self.set_decorated(False)
        self.set_keep_above(True)
        self.set_skip_taskbar_hint(True)
        self.set_skip_pager_hint(True)

        if pixmap != None:
            self.set_app_paintable(True)
            self.realize()
            self.window.set_back_pixmap(pixmap, False)

        messageLabel = gtk.Label('<span foreground="' + color +'">' \
                + escape(str(string)) + '</span>')
        messageLabel.set_use_markup(True)
        messageLabel.set_justify(gtk.JUSTIFY_CENTER)
        #messageLabel.set_line_wrap(True)
        messageLabel.set_ellipsize(pango.ELLIPSIZE_END)
        try:
            messageLabel.modify_font(pango.FontDescription(font))
        except e:
            print e

        if closePixmap == None:
            close = gtk.Label()
            close.set_label("<span background=\"#cc0000\" foreground=" \
                    + color + "\"> X </span>")
            close.set_use_markup(True)
        else:
            close = gtk.Image()
            close.set_from_pixbuf(closePixmap)

        closeEventBox = gtk.EventBox()
        closeEventBox.set_visible_window(False)
        closeEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        closeEventBox.connect("button_press_event", self.close)        
        closeEventBox.add(close)

        hbox = gtk.HBox()
        vbox = gtk.VBox()
        lbox = gtk.HBox()
        title = gtk.Label("")
        title.set_use_markup(True)

        avatarImage = gtk.Image()
        if userPixbuf != None:
            userPixbuf = resizePixbuf(userPixbuf, 48, 48)
            avatarImage.set_from_pixbuf(userPixbuf)

        lboxEventBox = gtk.EventBox()
        lboxEventBox.set_visible_window(False)
        lboxEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        lboxEventBox.connect("button_press_event", self.onClick, params)
        lboxEventBox.add(lbox)

        hbox.pack_start(title, True, True)
        hbox.pack_end(closeEventBox, False, False)
        lbox.pack_start(avatarImage, False, False, 10)
        lbox.pack_start(messageLabel, True, True, 5)

        vbox.pack_start(hbox, False, False)
        vbox.pack_start(lboxEventBox, True, True)

        self.grow(offset, False)
        self.add(vbox)
        vbox.show_all()

    def onClick(self, widget, event, params):
        if event.button == 1 and self.callback != None:
            self.callback(params)
        self.close()

    def resize(self):
        ''' change the size and position '''

        if self.scroll == 0: 
            if self.corner == 0 or self.corner == 2:
                l = self.offset
            else:
                l = gtk.gdk.screen_width() - self.offset - self.width

            if self.corner == 0 or self.corner == 1:
                t = 0
            else:
                t = gtk.gdk.screen_height() - self.height
        else:
            if self.corner == 0 or self.corner == 2:
                l = 0
            else:
                l = gtk.gdk.screen_width() - self.width

            if self.corner == 0 or self.corner == 1:
                t = self.offset
            else:
                t = gtk.gdk.screen_height() - self.offset - self.height

        gtk.Window.move(self, l, t)
        gtk.Window.resize(self, self.width, self.height)

    def show(self):
        ''' show it '''

        gtk.Window.show(self)

    def close(self , *args):
        ''' hide the Notification '''

        self.hide()

    def grow(self, offset, animate=True):
        ''' increase the size of the notification and position '''

        if animate and offset < self.offset:
            self.offset -= growFactor
            if offset > offset:
                self.offset = offset
        else:
            self.offset = offset

        if self.scroll == 0:
            if self.width < self.max:
                if self.width > self.max:
                    self.width = self.max
                else:
                    self.width += growFactor
        else:
            if self.height < self.max:
                if self.height + growFactor > self.max:
                    self.height = self.max
                else:
                    self.height += growFactor

        self.resize()

    def getOffset(self):
        ''' returns next notifications offset '''

        if self.scroll == 0: 
            if self.corner == 0 or self.corner == 2:
                return self.get_position()[0] + self.width
            else:
                return gtk.gdk.screen_width() - self.get_position()[0]
        else:
            if self.corner == 0 or self.corner == 1:
                return self.get_position()[1] + self.height
            else:
                return gtk.gdk.screen_height() - self.get_position()[1]

class MainClass(Plugin.Plugin):
    '''
    The notification plugin
    '''

    def __init__(self, controller, msn):
        ''' Constructor '''

        Plugin.Plugin.__init__(self, controller, msn, 1000)
        self.theme = controller.theme

        self.config = controller.config

        self.description = _('Show a little window in the bottom-right corner of the screen when someone gets online etc.')
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _('Notification')
        self.name = 'Notification'

        self.config.readPluginConfig(self.name)
        self.controller = controller

        self.filename = self.config.getPluginValue(self.name, 'filename', PATH.DEFAULT_THEME_PATH + 'guif.png')
        self.fontname  = self.config.getPluginValue(self.name, 'fontname', 'Sans')
        self.fontcolor = self.config.getPluginValue(self.name, 'fontcolor', '#000000') 

        self.notifyOnline  = int(self.config.getPluginValue(self.name, 'online', '1'))
        self.notifyOffline = int(self.config.getPluginValue(self.name, 'offline', '1'))
        self.notifyNewMail = int(self.config.getPluginValue(self.name, 'newMail', '1'))
	self.notifyNewMsg = int(self.config.getPluginValue(self.name, 'newMsg', '1'))
        self.notifyTyping = int(self.config.getPluginValue(self.name, 'typing', '0'))
        self.notifyStarted = int(self.config.getPluginValue(self.name, 'started', '0'))
        self.notifyIdle = int(self.config.getPluginValue(self.name, 'idle', '1'))
        self.position = int(self.config.getPluginValue(self.name, 'position', '3'))
        self.scroll = int(self.config.getPluginValue(self.name, 'scroll', '1'))

        self.onlineId = None
        self.offlineId = None
        self.newMsgId = None
        self.offMsgId = None
        self.typingId = None
        self.newMailId = None
        self.initMailId = None

    def notifyEnabled(self, contact = None):
        '''checks if notifications are enabled'''

        if self.notifyIdle and self.msn.status == 'BSY':
            return False
        if contact != None:
            if self.controller.contacts.get_blocked(contact):
                return False
        return True

    def online(self, msnp, email, oldStatus):
        ''' called when someone get online '''

        if not (self.notifyOnline and self.notifyEnabled(email)):
            return

        if oldStatus != 'FLN':
            return

        nick = unescape(self.controller.unifiedParser.getParser\
                (self.msn.getUserDisplayName(email)).get())
        contact = self.msn.contactManager.getContact(email)
        userPixbuf = self.theme.getUserDisplayPicture(contact)

        self.notificationManager.newNotification(unicode(nick)[:20] \
                + "\n" + _("is online"), self.position, self.scroll, \
                self.pixmap, self.close, self.startConversation, \
                (email, None), userPixbuf, self.fontname, self.fontcolor)

    def offline(self, msnp, email):
        ''' called when someone get offline '''

        if not (self.notifyOffline and self.notifyEnabled(email)):
            return

        nick = unescape(self.controller.unifiedParser.getParser\
                (self.msn.getUserDisplayName(email)).get())
        contact = self.msn.contactManager.getContact(email)
        userPixbuf = self.theme.getUserDisplayPicture(contact)

        self.notificationManager.newNotification(unicode(nick)[:20] \
                + "\n" + _("is offline"), self.position, self.scroll, \
                self.pixmap, self.close, None, None, userPixbuf, \
                self.fontname, self.fontcolor)

    def newMsg(self, msnp, email):
        '''called when someone send a message'''

        if not (self.notifyNewMsg and self.notifyEnabled(email)):
            return

	result = self.controller.conversationManager.getOpenConversation(email)
        if result != None:
            if self.notifyStarted:
                return
            window, conversation = result
            windowFocus = window.is_active()
            tabFocus = (window.conversation == conversation)
            if windowFocus and tabFocus:
                return

        nick = unescape(self.controller.unifiedParser.getParser\
                (self.msn.getUserDisplayName(email)).get())
        contact = self.msn.contactManager.getContact(email)
        userPixbuf = self.theme.getUserDisplayPicture(contact)

        self.notificationManager.newNotification(unicode(nick)[:20] \
                + "\n" + _('has send a message'), self.position, self.scroll, \
                self.pixmap, self.close, self.startConversation, \
                (email, None), userPixbuf, self.fontname, self.fontcolor)

    def offMsg(self, msnp, oim):
        '''called when someone send an offline message'''
        email = oim[0]['addr']

        if not (self.notifyNewMsg and self.notifyEnabled(email)):
            return

	result = self.controller.conversationManager.getOpenConversation(email)
        if result != None:
            window, conversation = result
            windowFocus = window.is_active()
            tabFocus = (window.conversation == conversation)
            if windowFocus and tabFocus:
                return

        nick = unescape(self.controller.unifiedParser.getParser\
                (self.msn.getUserDisplayName(email)).get())
        contact = self.msn.contactManager.getContact(email)
        userPixbuf = self.theme.getUserDisplayPicture(contact)

        self.notificationManager.newNotification(unicode(nick)[:20] \
                + "\n" + _('sent an offline message'), self.position, \
                self.scroll, self.pixmap, self.close, self.startConversation, \
                (email, None), userPixbuf, self.fontname, self.fontcolor)

    def receiveTyping(self, msn, switchboard, signal, args):
        '''called when someone starts typing'''
        email = args[0]

        if not (self.notifyTyping and self.notifyEnabled(email)):
            return

        if self.controller.conversationManager.getOpenConversation\
                (email, switchboard) != None:
            return

        nick = unescape(self.controller.unifiedParser.getParser\
                (self.msn.getUserDisplayName(email)).get())
        contact = self.msn.contactManager.getContact(email)
        userPixbuf = self.theme.getUserDisplayPicture(contact)

        self.notificationManager.newNotification(unicode(nick)[:20] \
                + "\n" + _('starts typing...'), self.position, self.scroll, \
                self.pixmap, self.close, self.startConversation, \
                (email, switchboard), userPixbuf, self.fontname, self.fontcolor)

    def newMail(self, msnp, From, FromAddr, Subject, MessageURL, PostURL, id):
        ''' called when receiving mail '''

        if not (self.notifyNewMail and self.notifyEnabled(FromAddr)):
            return

        contact = self.msn.contactManager.getContact(FromAddr)
        if contact == None:
            text = _('From: ') + From +' &lt;' + FromAddr + '&gt;'
            userPixbuf = None
        else:
            text = _('From: ') + unescape(self.controller.unifiedParser.\
                getParser(self.msn.getUserDisplayName(FromAddr)).get())
            userPixbuf = self.theme.getUserDisplayPicture(contact)

        text += '\n' + _('Subj: ') + escape(Subject)

        self.notificationManager.newNotification(_('New email') \
                + "\n" + text, self.position, self.scroll, self.pixmap, \
                self.close, self.openMail, (MessageURL, PostURL, id), \
                userPixbuf, self.fontname, self.fontcolor)

    def initMail(self, msnp):
        ''' called when receiving initial mail count '''

        if self.notifyNewMail:
            unread = self.controller.getUnreadMails()
            if unread > 0:
                if unread == 1:
                    s = ''
                else:
                    s = 's'
                params = {'num': unread, 's': s}
                self.notificationManager.newNotification(
                        _('You have %(num)i unread message%(s)s') % params, \
                        self.position, self.scroll, self.pixmap, self.close, \
                        self.openMail, (None, None, '2'), None, \
                        self.fontname, self.fontcolor)

    def startConversation(self, params):
        self.controller.newConversation(None, params[0], params[1], True)

    def openMail(self, params):
        desktop.open(self.controller.hotmail.getLoginPage\
                (params[0], params[1], params[2]))

    def start(self):
        ''' start the plugin '''

        self.notificationManager = NotificationManager(128, 200)
        self.onlineId = self.connect('user-online', self.online)
        self.offlineId = self.connect('user-offline', self.offline)
        self.newMsgId = self.connect('message-received', self.newMsg)
        self.offMsgId = self.connect('offline-message-received', self.offMsg)
        self.typingId = self.connect('switchboard::typing', self.receiveTyping)
        self.newMailId = self.connect('new-mail-notification', self.newMail)
        self.initMailId = self.connect\
                ('initial-mail-notification', self.initMail)

        try:
            self.pixmap, mask = gtk.gdk.pixbuf_new_from_file\
                    (self.filename).render_pixmap_and_mask()
        except:
            self.pixmap = None

        try:
            self.close = self.theme.getImage('close')
        except:
            self.close = None

        self.enabled = True

    def stop(self):    
        ''' stop the plugin '''

        self.disconnect(self.onlineId)
        self.disconnect(self.offlineId)
        self.disconnect(self.newMsgId)
        self.disconnect(self.offMsgId)
        self.disconnect(self.typingId)
        self.disconnect(self.newMailId)
        self.disconnect(self.initMailId)
        self.enabled = False
        self.notificationManager.closeAll()

    def check(self):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> (True , 'some message')
        else -> (False , 'error message')
        '''

        return (True, 'Ok')

    def configure(self):

        def _on_configure_ok(filename, font, color, online, offline, \
                newMsg, typing, newMail, started, idle, position, scroll):

            self.filename = filename
            self.fontname = font
            self.fontcolor = color
            self.notifyOnline = online
            self.notifyOffline = offline
            self.notifyNewMsg = newMsg
            self.notifyTyping = typing
            self.notifyNewMail = newMail
            self.notifyStarted = started
            self.notifyIdle = idle
            self.position = position
            self.scroll = scroll

            self.config.setPluginValue(self.name, 'filename', filename)
            self.config.setPluginValue(self.name, 'fontname', font)
            self.config.setPluginValue(self.name, 'fontcolor', color)
            self.config.setPluginValue(self.name, 'online', online)
            self.config.setPluginValue(self.name, 'offline', offline)
            self.config.setPluginValue(self.name, 'newMsg', newMsg)
            self.config.setPluginValue(self.name, 'typing', typing)
            self.config.setPluginValue(self.name, 'newMail', newMail)
            self.config.setPluginValue(self.name, 'started', started)
            self.config.setPluginValue(self.name, 'idle', idle)
            self.config.setPluginValue(self.name, 'position', position)
            self.config.setPluginValue(self.name, 'scroll', scroll)

            try:
                self.pixmap, mask = gtk.gdk.pixbuf_new_from_file\
                        (filename).render_pixmap_and_mask()
            except:
                self.pixmap = None

        PixmapDialog(_on_configure_ok, self.filename, \
                self.fontname, self.fontcolor, self.notifyOnline, \
                self.notifyOffline, self.notifyNewMsg, self.notifyTyping, \
                self.notifyNewMail, self.notifyStarted, self.notifyIdle, \
                self.position, self.scroll).show()

        return True



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''a module that defines the api of objects that display dialogs'''

import gettext

_ = gettext.gettext

def error(message, response_cb=None, title=_("Error!")):
    '''show an error dialog displaying the message, this dialog should
    have only the option to close and the response callback is optional
    since in few cases one want to know when the error dialog was closed,
    but it can happen, so return stock.CLOSE to the callback if its set'''
    pass

def warning(message, response_cb=None, title=_("Warning")):
    '''show a warning dialog displaying the messge, this dialog should
    have only the option to accept, like the error dialog, the response
    callback is optional, but you have to check if it's not None and
    send the response (that can be stock.ACCEPT or stock.CLOSE, if
    the user closed the window with the x)'''
    pass

def information(message, response_cb=None, title=_("Information"),):
    '''show a warning dialog displaying the messge, this dialog should
    have only the option to accept, like the error dialog, the response
    callback is optional, but you have to check if it's not None and
    send the response (that can be stock.ACCEPT or stock.CLOSE, if
    the user closed the window with the x)'''
    pass

def exception(message, response_cb=None, title=_("Exception"),):
    '''show the message of an exception on a dialog, useful to
    connect with sys.excepthook'''
    pass

def yes_no(message, response_cb, title=_("Confirm"), *args):
    '''show a confirm dialog displaying a question and two buttons:
    Yes and No, return the response as stock.YES or stock.NO or
    stock.CLOSE if the user closes the window'''
    pass

def yes_no_cancel(message, response_cb, title=_("Confirm"), *args):
    '''show a confirm dialog displaying a question and three buttons:
    Yes and No and Cancel, return the response as stock.YES, stock.NO,
    stock.CANCEL or stock.CLOSE if the user closes the window'''
    pass

def accept_cancel(message, response_cb, title=_("Confirm"), *args):
    '''show a confirm dialog displaying information and two buttons:
    Accept and Cancel, return stock.ACCEPT, stock.CANCEL or 
    stock.CLOSE'''
    pass

def contact_added_you(accounts, response_cb, title=("User invitation")):
    '''show a dialog displaying information about users
    that added you to their userlists, the accounts parameter is
    a tuple of mails that represent all the users that added you,
    the way you confirm (one or more dialogs) doesn't matter, but
    you should call the response callback only once with a tuple
    like: ((mail1, stock.YES), (mail2, stock.NO), (mail3, stock.CANCEL))
    YES means add him to your userlist, NO means block him, CANCEL
    means remind me later.'''
    pass

def add_contact(groups, group, response_cb, title=_("Add user")):
    '''show a dialog asking for an user address, and (optional)
    the group(s) where the user should be added, and the group selected
    by default (None to select no group) the response callback
    receives the response type (stock.ADD, stock.CANCEL or stock.CLOSE)
    the account and a tuple of group names where the user should be 
    added (give a empty tuple if you don't implement this feature, 
    the controls are made by the callback, you just ask for the email, 
    don't make any control, you are just implementing a GUI! :P'''
    pass

def add_group(response_cb, title=_("Add group")):
    '''show a dialog asking for a group name, the response callback
    receives the response (stock.ADD, stock.CANCEL, stock.CLOSE)
    and the name of the group, the control for a valid group is made
    on the controller, so if the group is empty you just call the
    callback, to make a unified behaviour, and also, to only implement
    GUI logic on your code and not client logic
    cb args: response, group_name'''
    pass

def set_nick(nick, response_cb, title=_("Change nick")):
    '''show a dialog asking for a new nick and displaying the current
    one, the response_cb receives the old nick, the new nick, 
    and the response (stock.ACCEPT, stock.CANCEL or stock.CLOSE)
    cb args: response, old_nick, new_nick'''
    pass

def set_message(message, response_cb, title=_("Change personal message")):
    '''show a dialog asking for a new personal message and displaying 
    the current one, the response_cb receives the old personal message
    , the new personal message and the response 
    (stock.ACCEPT, stock.CANCEL or stock.CLOSE)
    cb args: response, old_pm, new_pm'''
    pass

def set_picture(path_current, response_cb, title=_("Change picture")):
    '''show a dialog asking for the display picture and return the selected
    one, path_current is the path of the current selected picture.
     Return the new path or None if no new picture is selected'''
    pass

def set_custom_emoticon(ce_dir, response_cb):
    '''show a dalog to create a custome emotion, ce_dir is the
    directory where the image chooser dialog will open'''
    pass

def rename_group(name, response_cb, title=_("Rename group")):
    '''show a dialog with the group name and ask to rename it, the
    response callback receives stock.ACCEPT, stock.CANCEL or stock.CLOSE
    the old and the new name.
    cb args: response, old_name, new_name
    '''
    pass

def set_contact_alias(account, alias, response_cb, title=_("Set alias")):
    '''show a dialog showing the current alias and asking for the new
    one, the response callback receives,  the response 
    (stock.ACCEPT, stock.CANCEL, stock.CLEAR <- to remove the alias 
    or stock.CLOSE), the account, the old and the new alias.
    cb args: response, account, old_alias, new_alias'''
    pass

def about_dialog(name, version, copyright, comments, license, website,
    authors, translators, logo_path):
    '''show an about dialog of the application:
    * title: the title of the window
    * name: the name of the appliaction
    * version: version as string
    * copyright: the name of the copyright holder
    * comments: a description of the application
    * license: the license text
    * website: the website url
    * authors: a list or tuple of strings containing the contributors
    * translators: a string containing the translators
    '''
    pass



# -*- coding: utf-8 -*-
#
#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#   The Graphics Interchange Format(c) is the Copyright property of
#   CompuServe Incorporated. GIF(sm) is a Service Mark property of
#   CompuServe Incorporated.
#
# The unisys/lzw patent has expired, yes. If anyone puts another patent
# over this code, you must *burn* this file.

'''pygif: gif implementation in python

I (dx) have designed this to encode the handwritten messages for emesene.
While learning the format, I had to write that simple decoder -- so, it isn't
a complete gif89a decoder. If anyone needs to use it, contact me for help:
dx dxzone com ar.

Special credit for Jan de Mooij that fixed the giflzw encoder.'''


import struct
import math


KNOWN_FORMATS = ('GIF87a', 'GIF89a')


class Gif(object):
    '''Base class to encoder and decoder'''
    
    # struct format strings
        
    #17,18:
    FMT_HEADER = '<6sHHBBB'
    #20:
    FMT_IMGDESC = '<HHHHB'

    IMAGE_SEPARATOR = 0x2C
    EXTENSION_INTRODUCER = 0x21
    GIF_TRAILER = 0x3b

    LABEL_GRAPHIC_CONTROL = 0xF9
    LABEL_COMMENT = 0xFE
    LABEL_PLAINTEXT = 0x01

    FMT_EXT_GRAPHIC_CONTROL = '<BBHB' #89a
    
    def __init__( self, data, debug ):
        self.data = data
        self.pointer = 0
        
        # default data for an empty file
        self.header = 'GIF87a'
        self.ls_width = 0
        self.ls_height = 0
        self.flags = 0
        self.color_resolution = 0
        self.sort_flag = 0
        self.color_table_flag = 0
        self.global_color_table_size = 0
        self.background_color = 0
        self.aspect_ratio = 0
        # greyscale pallete by default
        self.pallete = [(x, x, x) for x in range(0, 256)]
        self.images = []

        self.debug_enabled = debug
        
    def pop( self, length=1 ):
        '''gets the next $len chars from the data stack,
        and increment the pointer'''
        
        start = self.pointer
        end = self.pointer + length
        self.pointer += length

        return self.data[start:end]
        
    def pops( self, format ):
        '''pop struct: get size, pop(), unpack()'''
        size = struct.calcsize(format) 
        return struct.unpack( format, self.pop(size) )

    def push( self, newdata ):
        '''adds $newdata to the data stack'''
        if self.debug_enabled:
            print repr(newdata), len(newdata)
        self.data += newdata
    
    def pushs( self, format, *vars ):
        '''push struct: adds a packed struct to the data stack'''
        self.push(struct.pack(format, *vars))
        
    def pop_bits( self, length ):
        '''return a list of bytes represented as bit lists'''
        bytes = self.pops( '<' + str(length) + 'B' )
        plain = []
        for byte in bytes:
            for bit in get_bits(byte):
                plain.append(bit)
        return plain
        
    def print_info( self ):
        '''prints out some useful info (..debug?)'''
        
        print "Version: %s" % self.header
        print "Logical screen width: %d" % self.ls_width
        print "Logical screen height: %d" % self.ls_height
        print "Flags: %s" % repr(self.flags)
        print " "*6,"Color resolution: %d" % self.color_resolution
        print " "*6,"Sort flag: %s" % str(self.sort_flag)
        print " "*6,"Global color table flag: %s" % str(self.color_table_flag)
        print " "*22,"...size: %d (%d bytes)" % \
            (self.global_color_table_size, self.global_color_table_size * 3)
        print "Background color: %d" % self.background_color
        print "Aspect ratio info: %d" % self.aspect_ratio

    def new_image( self, header=None ):
        '''adds a new image descriptor'''
        image = ImageDescriptor(self, header)
        self.images.append(image)
        return image
    
class ImageDescriptor(object):
    '''A class that represents a single image'''

    def __init__( self, parent, header=None ):
        # this will be set when needed
        self.codesize = 0

        # compressed output codes
        self.lzwcode = ''
        
        # uncompressed pixels (decoded)
        self.pixels = []
        
        # we assume a "fullscreen" image
        self.left = self.top = 0
        self.width = parent.ls_width
        self.height = parent.ls_height
        
        # yes, these default flags work...
        self.flags = [False for x in range(8)]
        self.local_color_table_flag = False
        self.interlace_flag = False
        self.sort_flag = False
        self.local_color_table_size = 0
        self.pallete = []
        
        if header:
            self.setup_header(header)

    def setup_header( self, header ):
        '''takes a header tuple and fills the attributes'''

        self.left = header[0]
        self.top = header[1]
        self.width = header[2]
        self.height = header[3]

        self.flags = get_bits( header[4] )
        
        self.local_color_table_flag = self.flags[7]
        assert self.local_color_table_flag == False, \
            "Local color tables not implemented" # TODO
        self.interlace_flag = self.flags[6]
        self.sort_flag = self.flags[5]
        #-- flags 4 and 3 are reserved
        self.local_color_table_size =  2 ** (pack_bits(self.flags[:2]) + 1)


    def get_header(self):
        '''builds a header dynamically'''
        flags = [False for x in range(8)]
        flags[7] = self.local_color_table_flag
        flags[6] = self.interlace_flag
        flags[5] = self.sort_flag
        
        # useless!
        flags[2], flags[1], flags[0] = get_bits(len(self.pallete), bits=3)

        return (self.left, self.top, self.width, self.height, pack_bits(flags))

    header = property(fget=get_header)
        

class GifDecoder( Gif ):
    '''decodes a gif file into.. something.. else..'''
    def __init__( self, data, debug=False ):
        Gif.__init__( self, data, debug )
        self.fill()
        
    def fill( self ):
        '''reads the data and fills each field of the file'''
        
        # start reading from the beggining of the file
        self.pointer = 0
        
        #17. Header.
        #18. Logical Screen Descriptor.
        data = self.pops( Gif.FMT_HEADER )
        
        self.header = data[0]
        self.ls_width = data[1]
        self.ls_height = data[2]
        self.background_color = data[4]
        self.aspect_ratio = data[5]
        
        # flags field
        self.flags = get_bits( data[3] )
        #1 bit
        self.color_table_flag = self.flags[7]
        self.sort_flag = self.flags[3]
        #3 bit
        self.color_resolution = pack_bits(self.flags[4:7]) # 7 not included
        #3 bit
        self.global_color_table_size = 2 ** (pack_bits(self.flags[:3]) + 1)
        
        #19. Global Color Table.
        if self.color_table_flag:
            size = (self.global_color_table_size) * 3
            self.pallete = self.get_color_table(size)
        else:
            # generate a greyscale pallete
            self.pallete = [(x, x, x) for x in range(256)]
        
        # blocks
        while True:
            try:
                nextbyte = self.pops('<B')[0]
            except:
                nextbyte = 0x3b # force end
            
            #20. Image Descriptor
            if nextbyte == Gif.IMAGE_SEPARATOR:
                descriptor = self.pops(Gif.FMT_IMGDESC)
                image = self.new_image(descriptor)
                image.codesize = self.pops('<B')[0]
                image.lzwcode = ''

                while True:
                    try:
                        blocksize = self.pops('<B')[0]
                    except:
                        break
                    if blocksize == 0:
                        break   # no more image data
                    lzwdata = self.pop(blocksize)
                    image.lzwcode += lzwdata
                
                if self.debug_enabled:
                    print 'LZW length:', len(image.lzwcode)

                image.pixels = self.lzw_decode(image.lzwcode, image.codesize, \
                    self.global_color_table_size)
            
            # Extensions
            elif nextbyte == Gif.EXTENSION_INTRODUCER:
                pass
            # Gif trailer
            elif nextbyte == Gif.GIF_TRAILER:
                return
            
            # "No Idea What Is This"
            else:
                pass

    def get_color_table( self, size ):
        '''Returns a color table in the format [(r,g,b),(r,g,b), ...]'''
         
        raw_color_table = self.pops("<%dB" % size)
        pos = 0
        pallete = []
        
        while pos + 3 < (size+1):
            red = raw_color_table[pos]
            green = raw_color_table[pos+1]
            blue = raw_color_table[pos+2]
            pallete.append((red, green, blue))
            pos += 3
        return pallete

    def lzw_decode(self, input, initial_codesize, color_table_size):
        '''Decodes a lzw stream from input
        Returns list of ints (pixel values)'''
        string_table = {}
        output = []
        old = ''
        index = 0
        
        codesize = initial_codesize + 1
        clearcode, end_of_info = color_table_size, color_table_size + 1
        bits = string_to_bits(input)

        def pop(size):
            '''Pops <size> bits from <bits>'''
            out = []
            for i in range(size):
                out.append(bits.pop(0))
            return out
        
        def clear():
            '''Called on clear code'''
            string_table.clear()
            for index in range(color_table_size):
                string_table[index] = chr(index)
            index = end_of_info + 1
            return index
        
        index = clear()
        
        # skip first (clear)code
        bits = bits[codesize:]

        # read first code, append to output
        code = bits_to_int(pop(codesize))
        output = [ord(string_table[code])]
        
        old = string_table[code]
        
        while len(bits) > 0:
            # read next code
            code = bits_to_int(pop(codesize))

            # special code?
            if code == clearcode:
                index = clear()
                
                codesize = initial_codesize + 1
                code = bits_to_int(pop(codesize))
                
                output.append(ord(string_table[code]))
                old = string_table[code]
                continue
                   
            elif code == end_of_info:
                break
            
            # code in stringtable?
            if code in string_table:
                c = string_table[code]
                string_table[index] = old + c[0]
            else:
                c = old + old[0]
                string_table[code] = c
     
            index += 1
            old = c
            output += [ord(x) for x in c]
            
            if index == 2 ** codesize:
                codesize += 1
                if codesize == 13:
                    codesize = 12
                    print 'decoding error, missed a clearcode?'
                    print 'index:', index
                    #exit()
            
        if self.debug_enabled:
            print 'Output stream len: %d' % len(output)
        return output

class GifEncoder(Gif):
    '''Encodes a *something* into a gif'''
    
    def __init__( self, basedecoder=None, debug=False ):
        Gif.__init__( self, '', debug )
        if basedecoder:
            self.clone(basedecoder)

    def clone( self, decoder ):
        '''moves decoder data into this class'''
        self.header = decoder.header
        self.ls_width = decoder.ls_width
        self.ls_height = decoder.ls_height
        self.flags = decoder.flags
        self.background_color = decoder.background_color
        self.aspect_ratio = decoder.aspect_ratio

        self.color_table_flag = decoder.color_table_flag
        self.pallete = decoder.pallete
        self.color_resolution = decoder.color_resolution
        self.global_color_table_size = decoder.global_color_table_size
        
        self.images = decoder.images

    def write( self, filename ):
        '''rebuilds the gif stream and writes it to filename'''
        self.data = ''
        if self.debug_enabled:
            print "header, screen descriptor"
        #17. Header.
        #18. Logical Screen Descriptor.
        self.pushs( Gif.FMT_HEADER, self.header, self.ls_width, \
            self.ls_height, pack_bits(self.flags), \
            self.background_color, self.aspect_ratio )
        
        if self.debug_enabled:
            print "global color table"
        #19. Global Color Table.
        #print self.color_table_flag, self.pallete
        if self.color_table_flag:
            for red, green, blue in self.pallete:
                self.pushs("<BBB", red, green, blue)

        if self.debug_enabled:
            print "images"
        #20. Image Descriptor
        self.images = self.images[:1]
        for image in self.images:
            if self.debug_enabled:
                print "image:", image
            self.pushs('<B', Gif.IMAGE_SEPARATOR)
            self.pushs(Gif.FMT_IMGDESC, *image.header)
            self.pushs('<B', image.codesize)
            lzwcode = image.lzwcode
            while len(lzwcode) > 0:
                chunk, lzwcode = lzwcode[:254], lzwcode[254:]
                self.pushs('<B', len(chunk))
                self.push(chunk)

        # Trailer, end of file
        self.pushs('<B', 0)
        self.pushs('<B', Gif.GIF_TRAILER)
        
        rc = open(filename, 'wb')
        rc.write(self.data)
        
    def build_flags(self):
        '''builds the flag integer'''
        flaglist = [False for x in range(8)]
        # single bit flags
        flaglist[7] = bool(self.color_table_flag)
        flaglist[3] = bool(self.sort_flag)
        
        # color resolution
        colorbits = get_bits(self.color_resolution, bits=3)
        if self.debug_enabled:
            print 'reso: %d' % self.color_resolution
        flaglist[4], flaglist[5], flaglist[6] = colorbits

        if self.color_table_flag:
            # global color table size
            if self.debug_enabled:
                print 'pallete: %d' % len(self.pallete)
                print 'FIXME: is there an easier way?'
            val = int(math.ceil(math.log(len(self.pallete))/math.log(2)-1))
            gctsbits = get_bits(val, bits=3)
            flaglist[0], flaglist[1], flaglist[2] = gctsbits
        
        # self.flags is a list, not an int
        self.flags = flaglist

    def lzw_encode(self, pixels):
        '''Encodes a pixel stream (list of ints) in lzw code
        Returns tuple (initial code size, lzw string)'''

        if self.debug_enabled:
            print 'encoding %d pixels...' % len(pixels)
        
        string_table = {}
        output = []
        old = ''
        index = 0
        
        color_table_size = self.global_color_table_size
        clearcode = color_table_size
        end_of_info = color_table_size + 1
        # what?
        initial_codesize = \
            int(math.ceil( math.log(color_table_size) / math.log(2) - 1 )) + 1
        
        codesize = initial_codesize + 1
        bits = []

        def clear():
            string_table.clear()
            for index in range(color_table_size):
                string_table[chr(index)] = index
            index = end_of_info + 1
            return index
        
        # Initialize code table
        index = clear()
        #for i in string_table: print ord(i), '-', string_table[i]
        
        # add clear code
        add = get_bits(clearcode, bits=codesize)
        output += add
        
        c = chr(pixels.pop(0))
        counter = 0
        maxcount = len(pixels)
        
        while counter < maxcount:
            p = chr(pixels[counter])
            s = c + p
            
            counter += 1
            
            if s in string_table:
                c += p
            else:
                string_table[s] = index
                index += 1
                
                output += get_bits(string_table[c], bits=codesize)
                c = p
                
                if index == ((2 ** codesize)+1):
                    codesize += 1
                    if codesize == 13:
                        output += get_bits(clearcode, bits=12)
                        index = clear()
                        codesize = initial_codesize + 1

        if c != '':
            add = get_bits(string_table[c], bits=codesize)
            output += add

        add = get_bits(end_of_info, bits=codesize)
        output += add
        o = bits_to_string(output)
        return (initial_codesize, o)


def bits_to_int(bits):
    '''high level bit list packer'''
    i = 0
    c = 0
    bits.reverse()
    while c < len(bits):
        if bits[c]:
            i += 2 ** (len(bits) - c - 1)
        c += 1
    return i

def get_bits( flags, reverse=False, bits=8 ):
    '''return a list with $bits items, one for each enabled bit'''

    mybits = [ 1 << x for x in range(bits) ]
    
    ret = []
    for bit in mybits:
        ret.append(flags & bit != 0)

    if reverse:
        ret.reverse()

    return ret

def pack_bits( bits ):
    '''convert a bit (bool or int) tuple into a int'''
    packed = 0
    level = 0
    for bit in bits:
        if bit:
            packed += 2 ** level    
        #packed += int(bit) << level
        #print bit, packed, level
        level += 1
    return packed

def string_to_bits(string):
    '''high level string unpacker'''
    bits = []
    for byte in string:
        for bit in get_bits(ord(byte)):
            bits.append(bit)
    return bits

def bits_to_string(bits):
    '''high level bit list packer'''
    string = ''
    while len(bits)>0:
        code = pack_bits(bits[:8])
        bits = bits[8:]
        string += chr(code)
    return string

def readable(bool_list):
    '''Converts a list of booleans to a readable list of ints
    Useful for debug only'''
    return [int(x) for x in bool_list]




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import tempfile
import threading
import time

import Plugin

class MainClass(Plugin.Plugin):
    '''Main plugin class'''

    def __init__( self, controller, msn ):
        '''Constructor'''

        Plugin.Plugin.__init__( self, controller, msn )
        self.description = _( 'Take screenshots and send them with ' +
                    '/screenshot [save] [<seconds>]' )
        self.authors = { 'Dx' : 'dx@dxzone.com.ar' }
        self.website = 'http://www.dxzone.com.ar'
        self.displayName = 'Screenshots'
        self.name = 'Screenshots'
        
        self.config = controller.config
        self.config.readPluginConfig(self.name)

        self.controller = controller
        self.slash = controller.Slash
        self.msn = msn
        
        if os.name == 'posix':
            self.root = controller.mainWindow.get_root_window()
        else:
            self.root = None

    def start( self ):
        '''start the plugin'''
        self.slash.register('screenshot', self.screenshot, \
            _('Takes screenshots') )

        self.enabled = True

    def screenshot( self, slashAction ):
        params = slashAction.getParams().split()
        
        save = False
        delay = 0
        
        # parse parameters
        for param in params:
            if param == 'save':
                save = True
            elif param.isdigit():
                delay = int(param)
        
        if delay > 0:
            slashAction.outputText(_('Taking screenshot in %s seconds') % delay)
        
        if not save:
            if self.config.getPluginValue(self.name, 'tip', '0') == '0':
                slashAction.outputText( \
                    _('Tip: Use "/screenshot save <seconds>" to skip the upload '))
                self.config.setPluginValue(self.name, 'tip', '1')
        
        ScreenThread( slashAction, self, save=save, delay=delay ).start()

    def stop( self ):    
        '''stop the plugin'''
        self.slash.unregister('screenshot')
        self.enabled = False

    def check( self ):
        return ( True, 'Ok' )

class ScreenThread(threading.Thread):

    def __init__(self, action, plugin, save=False, delay=0):
        threading.Thread.__init__( self )
        self.slashAction = action
        self.plugin = plugin
        self.save = save
        self.sleepTime = int(delay)

    def run(self):
        print "sleeping: " + str(self.sleepTime)
        time.sleep(self.sleepTime)
        if os.name == 'nt':
            print 'screenshot on windows never tested, check source!'

            # yes, i had to break this link.. 80 cols :P
            ## mail.python.org/pipermail/python-win32/
            ## .../2003-January/000709.html

            # import win32api, win32clipboard
            # win32api.keybd_event(win32con.VK_PRINT, 0)
            # win32clipboard.OpenClipboard(0)
            # data = win32clipboard.GetClipboardData(win32clipboard.CF_DIB)
            # win32clipboard.CloseClipboard()
        elif os.name == 'posix':
            temp = tempfile.mkstemp(prefix='screenshot', suffix='.png')
            os.close(temp[0])
            
            # überfast screenshots! 1.9secs in my pentium II 350mhz
            # er.. yes, that's fast.. try running import
            root = self.plugin.root
            size = root.get_size()
            colormap = root.get_colormap()
            pixmap = gtk.gdk.pixmap_foreign_new(root.xid)
            pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, *size)
            pixbuf.get_from_drawable(pixmap, colormap, 0, 0, 0, 0, *size)
            pixbuf.save(temp[1], 'png')
            
        if not self.save:
            self.slashAction.conversation.sendFile(temp[1])
        else:
            self.slashAction.outputText( _('Temporary file:') + ' ' + \
                temp[1] )




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''a class that represent a contextual menu of a contact'''

import Menu
import stock

class ContactMenu(Menu.Menu):
    '''class that represent a contextual menu of a contact'''

    def __init__(self, contact, contacts, dialog):
        '''class constructor'''
        Menu.Menu.__init__(self)

        self.contact = contact
        self.contacts = contacts
        self.dialog = dialog
        
        self._build()

    def _build(self):
        '''build the menu'''
        self.remove_item = Menu.Item(_('_Remove'), Menu.Item.TYPE_STOCK, 
            stock.REMOVE)
        self.remove_item.signal_connect('selected', self._on_remove_selected)
        
        self.block_item = Menu.Item(_('_Block'), Menu.Item.TYPE_STOCK, 
            stock.STOP)
        self.block_item.signal_connect('selected', self._on_block_selected)
        
        self.unblock_item = Menu.Item(_('_Unblock'), Menu.Item.TYPE_STOCK, 
            stock.APPLY)
        self.unblock_item.signal_connect('selected', self._on_unblock_selected)
        
        self.move_item = Menu.Item(_('_Move'), Menu.Item.TYPE_STOCK, 
            stock.FORWARD)
        self.move_item.signal_connect('selected', self._on_move_selected)
        
        self.move_item = Menu.Item(_('_Copy'), Menu.Item.TYPE_STOCK, 
            stock.COPY)
        self.move_item.signal_connect('selected', self._on_copy_selected)

        self.set_alias_item = Menu.Item(_('Set _alias'), Menu.Item.TYPE_STOCK, 
            stock.EDIT)
        self.set_alias_item.signal_connect('selected', 
            self._on_set_alias_selected)
        
        self.append(self.remove_item)
        self.append(self.block_item)
        self.append(self.unblock_item)
        self.append(self.set_alias_item)
        self.append(self.move_item)
    
    def _on_remove_selected(self, item):
        '''called when remove is selected'''
        def _yes_no_cb(response):
            '''callback from the confirmation dialog'''
            if response == stock.YES:
                self.contacts.remove(self.contact.account)

        self.dialog.yes_no(
            _('Are you sure you want to remove contact %s?') % \
            (self.contact.account,), _yes_no_cb)

    def _on_block_selected(self, item):
        '''called when block is selected'''
        self.contacts.block(self.contact.account)

    def _on_unblock_selected(self, item):
        '''called when unblock is selected'''
        self.contacts.unblock(self.contact.account)

    def _on_move_selected(self, item):
        '''called when move is selected'''
        pass

    def _on_copy_selected(self, item):
        '''called when move is selected'''
        pass

    def _on_set_alias_selected(self, item):
        '''called when set_alias is selected'''
        self.contacts.set_alias_dialog(self.contact.account)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import gobject
import base64 # yes base64, go out and look how gaim does it :P

import Widgets
import dialog
import emesenelib.common
from emesenecommon import PATH, sort


class Login(gtk.VBox):
    '''This class represents the login widget where you insert the user, 
    pass and status'''
    
    def __init__(self , controller, interface):
        '''Constructor'''
        
        gtk.VBox.__init__(self)
                
        self.controller = controller
        
        self.set_spacing(20)
        self.set_border_width(20)
        
        self.buildInterface(interface)

    def buildInterface(self, interface):
        self.currentInterface = interface
        
        lUser = gtk.Label(_('_User:'))
        lPass = gtk.Label(_('_Password:'))
        lStatus = gtk.Label(_('_Status:'))
        
        lUser.set_use_underline(True)
        lPass.set_use_underline(True)
        lStatus.set_use_underline(True)
        
        #we add the completion to the user entry
        
        self.tUser = gtk.combo_box_entry_new_text()
        self.userListStore = gtk.ListStore(gobject.TYPE_STRING, gtk.gdk.Pixbuf)
        self.userCompletion = gtk.EntryCompletion()
        #set up the completion on entry child
        self.tUser.get_children()[0].set_completion(self.userCompletion)
        self.userCompletion.set_model(self.userListStore)
        self.userPixbufCell = gtk.CellRendererPixbuf()
        self.userCompletion.pack_start(self.userPixbufCell)
        self.userCompletion.add_attribute(self.userPixbufCell, 'pixbuf', 1)
        # create a gtk.CellRendererText and pack it in the completion.
        # Also set the 'text' attribute
        self.userCompletion.set_text_column(0)
        self.userCompletion.connect('match-selected', self.matchSelected)
        self.tUser.connect("changed", self.on_comboxEntry_changed)
        self.tUser.connect("key-release-event", self.on_comboxEntry_keyrelease)
        
        self.statusListStore = gtk.ListStore(gtk.gdk.Pixbuf, 
                                              gobject.TYPE_STRING,
                                              gobject.TYPE_STRING)
        self.tStatus = gtk.ComboBox(self.statusListStore)
        self.statusPixbufCell = gtk.CellRendererPixbuf()
        self.statusTextCell = gtk.CellRendererText()
        self.tStatus.pack_start(self.statusPixbufCell, False)
        self.tStatus.pack_start(self.statusTextCell, False)
        self.statusPixbufCell.set_property('xalign', 0.0)
        self.statusPixbufCell.set_property('xpad', 5)
        self.statusTextCell.set_property('xalign', 0.0)
        self.statusTextCell.set_property('xpad', 5)
        self.tStatus.add_attribute(self.statusTextCell, 'text', 2)
        self.tStatus.add_attribute(self.statusPixbufCell, 'pixbuf', 0)
            
        self.tPass = gtk.Entry(128)
        self.tPass.set_visibility(False)
        self.tPass.connect('activate' , self.bLogin_clicked)
        
        lUser.set_mnemonic_widget(self.tUser)
        lPass.set_mnemonic_widget(self.tPass)
        lStatus.set_mnemonic_widget(self.tStatus)
        
        # fill the status combobox
        self.statusList = []
        j = 0
        for i in self.controller.status_ordered[ 1 ]:
            if i != 'offline':
                self.statusListStore\
                        .append([ self.controller.theme.statusToPixbuf(
                               emesenelib.common.status_table[i]), i,
                               _(self.controller.status_ordered[2][ j]) ])
                self.statusList.append(i)
                j += 1
                            
        self.tStatus.set_active(0)
        
        pixbuf = self.controller.theme.getImage('login', False)
        self.loginImage = Widgets.avatarHolder()
        self.loginImage.set_from_pixbuf(pixbuf)
        self.pack_start(self.loginImage , True , False)
        
        self.fieldsBox = gtk.VBox(spacing=10)
        
        userBox = gtk.VBox(spacing=4)
        lUser.set_alignment(0.0, 0.5)
        userBox.pack_start(lUser , False , False)
        userBox.pack_start(self.tUser , False , False)
    
        self.fieldsBox.pack_start(userBox, False, False)
        
        passBox = gtk.VBox(spacing=4)
        lPass.set_alignment(0.0, 0.5)
        passBox.pack_start(lPass , False , False)
        passBox.pack_start(self.tPass , False , False)
        
        self.fieldsBox.pack_start(passBox, False, False)
        
        statusBox = gtk.VBox(spacing=4)
        lStatus.set_alignment(0.0, 0.5)
        statusBox.pack_start(lStatus , False , False)
        statusBox.pack_end(self.tStatus, True, True)

        self.fieldsBox.pack_start(statusBox, False, False)
        
        fieldsAlig = gtk.Alignment(0.5, 0.5, 0.75, 0.0)
        fieldsAlig.add(self.fieldsBox)
        self.pack_start(fieldsAlig, True, False)
        
        buttonBox = gtk.HButtonBox()
        
        if interface == 'login':
            self.bLogin = gtk.Button(_('_Login'), gtk.STOCK_CONNECT)
            self.bLogin.connect('clicked' , self.bLogin_clicked)
            buttonBox.pack_start(self.bLogin, False, False)
            
            self.cRemember = gtk.CheckButton(_('_Remember me'), True)
            self.cRemember.connect('toggled' , self.on_cRemember_toggled)
            
            self.forgetMe = gtk.EventBox()
            self.forgetMe.set_events(gtk.gdk.BUTTON_PRESS_MASK)
            self.forgetMeLabel = gtk.Label('<span foreground="#0000AA">(' + \
                                            _('Forget me') + ')</span>')
            self.forgetMeLabel.set_use_markup(True)
            self.forgetMe.add(self.forgetMeLabel)
            self.forgetMe.connect('button_press_event', self.onForgetMe)
     
            self.cRememberPass = gtk.CheckButton(_('R_emember password'), True)
            self.cRememberPass.connect('toggled' , self.on_cRememberPass_toggled)

            # TODO: find a better description and get-text it
            self.cAutoLogin = gtk.CheckButton(_('Auto-Login'), True)
            self.cAutoLogin.connect('toggled' , self.on_cAutoLogin_toggled)
            
            if self.controller.config.glob['rememberMe']:
                self.cRemember.set_active(True)
                
            if self.controller.config.glob['rememberMyPassword']:
                self.cRememberPass.set_active(True)

            self.checkBox = gtk.VBox(spacing=4)
        
            rememberBox = gtk.HBox(spacing=4)
            rememberBox.pack_start(self.cRemember , False , False)
            rememberBox.pack_start(self.forgetMe , False , False)
        
            self.checkBox.pack_start(rememberBox, False, False)
            self.checkBox.pack_start(self.cRememberPass , False , False)
            self.checkBox.pack_start(self.cAutoLogin , False , False)
            
            checkAlig = gtk.Alignment(0.5, 0.5)
            checkAlig.add(self.checkBox)
            
            self.pack_start(checkAlig , True , False)

        elif interface == 'reconnect':
            self.fieldsBox.set_sensitive(False)
            
            self.lConnectionError = gtk.Label()
            self.lConnectionError.set_markup('<b>' + _('Connection error') + '</b>')
            
            self.lReconnectCounter = gtk.Label()
            
            self.bCancelReconnect = gtk.Button(_('Cancel'), gtk.STOCK_CANCEL)
            self.bCancelReconnect.connect('clicked', self.onCancelReconnect)
        
            self.pack_start(self.lConnectionError, True, False)
            self.pack_start(self.lReconnectCounter, True, False)
            buttonBox.pack_start(self.bCancelReconnect, False, False)
        elif interface == 'loading':
            self.fieldsBox.set_sensitive(False)

            pixbuf = self.controller.theme.getImage('loading', True)
            self.loading = gtk.image_new_from_animation(pixbuf)
        
            cancelButton = gtk.Button(_('Cancel'), gtk.STOCK_CANCEL)
            cancelButton.connect('clicked', self.onCancelLogin)
            buttonBox.pack_start(cancelButton, False, False)
            
            self.pack_start(self.loading, False, False)
            # XXX: progress stuff maybe?

        self.pack_start(buttonBox, True, False)            
        self.fillUserList()
        self.show_all()
        self.tUser.grab_focus()
    
    def setFieldValues(self, user, password, status):
        self.tUser.prepend_text(user)
        self.tUser.set_active(0)
        self.tPass.set_text(password)
        
        statusOrdered = self.controller.status_ordered[1]
        if status in statusOrdered:
            self.tStatus.set_active(statusOrdered.index(status))
                        
    def onCancelReconnect(self, *args):
        self.controller.cancelReconnect()

    def onCancelLogin(self,*args):
        self.controller.cancelLogin()
    
    def fillUserList(self):
        '''fill the user list to the completion'''
        
        # try to fill the user combo
        self.users = UserManager(os.path.join(PATH.CONFIG_DIR, 'users.dat'))

        j=0
        for i in sort(self.users.getUsers()):
            self.tUser.append_text(i)
            self.userListStore.append([ i , self.controller.theme.getImage('online') ])    
            if self.controller.config.glob['lastLoggedAccount'] == i:
                self.getPreferences(i)
                self.tUser.set_active(j)
            j+=1

    def on_cRemember_toggled(self , *args):
        self.controller.config.glob['rememberMe'] = self.cRemember.get_active()

    def on_cRememberPass_toggled(self , *args):
        if self.cRememberPass.get_active():
            self.controller.config.glob['rememberMyPassword'] = True
            self.cRemember.set_active(True)
            self.cRemember.set_sensitive(False)
            #self.tPass.set_sensitive(False)
        else:
            self.controller.config.glob['rememberMyPassword'] = False
            self.cRemember.set_sensitive(True)
            #self.tPass.set_sensitive(True)
        self.on_cRemember_toggled()

    def on_cAutoLogin_toggled(self , *args):
        if self.cAutoLogin.get_active():
            self.controller.config.glob['autoLogin'] = True
            self.cRememberPass.set_active(True)
            self.cRememberPass.set_sensitive(False)
        else:
            self.controller.config.glob['autoLogin'] = False
            self.cRememberPass.set_sensitive(True)
        self.on_cRememberPass_toggled()

    def setReconnectCounter(self, seconds):
        if seconds > 0:
            # FIXME: use %s instead
            self.lReconnectCounter.set_text(_('Reconnecting in ') + str(seconds) + _(' seconds'))
        else:
            self.lReconnectCounter.set_text(_('Reconnecting...'))

    def login(self):
        self.controller.login(self.getUser() , self.getPass() , self.getStatus())        

    def bLogin_clicked(self , *args):
        if self.getUser() == '' or self.getPass() == '' or self.getStatus() == '':
            dialog.error(_("User or password empty"))
            return
    
        if self.currentInterface == 'login' and self.cRemember.get_active():
            if self.cRememberPass.get_active():
                self.users.addUser(self.getUser(), base64.b16encode(self.getPass()), self.getStatus())
            else:
                self.users.addUser(self.getUser(),'',self.getStatus())
            
        self.users.save()
        self.login()
    
    def getUser(self):
        self.active_user = self.tUser.get_active()
    
        return self.tUser.get_active_text()
        
    def getPass(self):
        return self.tPass.get_text()
        
    def getStatus(self):
        return self.statusListStore.get(self.tStatus.get_active_iter(), 1)[ 0 ]
    
    def on_comboxEntry_changed(self, *args):
        self.getPreferences(self.getUser())
    
    def matchSelected(self, completion=None, model=None, iter=None):
        '''this callback is called when a user is selected with autocompletion
        then i check if the password is stored.'''
        userInEntry = model.get(iter, 0)[ 0 ]
        self.getPreferences(userInEntry)
    
    def getPreferences(self, user):
                
        password = self.users.getPassword(user)
        status = self.users.getStatus(user)
        avatar = self.getUserAvatarPath(user)      
        try:
            self.loginImage.set_from_file(avatar)
        except gobject.GError:
            pass
        except: # everything else, like open() problems
            pass
        
        if password != '':
            self.tPass.set_text(base64.b16decode(password))
        else:
            self.tPass.set_text('')
            if self.currentInterface == 'login': 
                self.cRememberPass.set_active(False)
        
        if status != '':
            try:
                self.tStatus.set_active(self.statusList.index(status) )
            except:
                pass
                
    def getUserAvatarPath(arg, user):
        user = user.replace('@', '_').replace('.', '_')
        try:
            s = open(os.path.join(PATH.CONFIG_DIR, user, 'config'), 'r').read()
            s.replace(' ', '')
            path = s.split('avatarPath=')[1].split('\n')[0]
        except Exception, e:
            path = PATH.DEFAULT_THEME_PATH + 'login.png'

        if os.path.isfile(path):
            return path

        return PATH.DEFAULT_THEME_PATH + 'login.png'
            
    def onForgetMe(self, *args):
        '''This method is called when the user press the forget me event box'''
        self.tUser.remove_text(self.active_user)
       
        self.users.removeUser(self.getUser())
        self.tUser.get_children()[0].set_text('')
        self.tPass.set_text('')
        self.users.save()
    
    def on_comboxEntry_keyrelease(self, widget, event):
        if event.keyval == gtk.keysyms.Tab:
            self.tPass.grab_focus()
            
        if event.keyval == gtk.keysyms.Return:
            self.login()
    
class UserManager(object):
    '''This class manages the file that hold the user:password data
    the format of users.dat is:
    mail:password:status
    the password is encrypted in a magic way :P'''
    
    def __init__(self, path):
        '''Contructor'''
        
        self.path = path
        self.users = {}
        
        self.load()
        
    def getUsers(self):
        '''return a list of users'''
        
        return self.users.keys()
    
    def getPassword(self, user):
        '''return the password of the user'''
        
        if self.users.has_key(user):
            return self.users[ user ][ 0 ]
        else:
            return ''
        
    def getStatus(self, user):
        '''return the status of the user'''
        
        if self.users.has_key(user):
            return self.users[ user ][ 1 ]
        else:
            return 'NLN'
    
    def addUser(self, user, password = '', status = 'online'):
        '''add a user to the dictionary'''
        
        # if no password provided and we already have an user saved
        # with a password, we keep it

        if password == '' and \
           self.users.has_key(user) and \
           self.users[ user ][ 0 ] != '':
            password = self.users[ user ][ 0 ]
        self.users[ user ] = (password, status)
        
    def removeUser(self, user):
        '''remove the user from the dictionary'''
        
        if self.users.has_key(user):
            del self.users[ user ]
            
    def save(self):
        '''save the values to the file'''
        
        users = file(self.path, 'w')
        
        for user in self.users.keys():
            users.write(user + ':')
            users.write(self.users[ user ][ 0 ] + ':')
            users.write(self.users[ user ][ 1 ] + '\n')
            
    def load(self):
        '''load the content of the file'''
        
        justCreated = False
        
        try:    
            usersList = open(self.path, 'r')
        except:
            usersList = open(self.path , 'w')
            justCreated = True

        if not justCreated:
            for line in usersList.readlines():
                (user,password,status) = line.strip().split(':')
                self.users[ user ] = (password, status)
        
        usersList.close()




# -*- coding: utf-8 -*-

#   This file is a plugin for emesene.
#
#    Dbus Emesene Plugin is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Dbus Emesene Plugin is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os

import Plugin
try:
    import dbus
    dbusError = ''
    if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
        import dbus.glib
        import dbus.service
        import dbus.mainloop.glib
except Exception, e:
    dbusError = e

BUS_NAME = 'org.emesene.dbus'
OBJECT_PATH = '/org/emesene/dbus'

class MainClass(Plugin.Plugin):
    '''Main plugin class'''
    
    def __init__(self, controller, msn):
        '''Contructor'''
        
        Plugin.Plugin.__init__(self, controller, msn)
        self.theme = controller.theme
        
        self.description = _('With this plugin you can interact via Dbus with emesene')
        self.authors = {'Roger Duran' : 'RogerDuran at gmail dot com'}
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = _('Dbus')
        self.name = 'Dbus'
        
        #callback ids
        self.user_connect_id = None
        self.user_offline_id = None
        self.self_nick_changed_id = None
        self.psm_changed_id = None

        self.dbus_object = None
        
    def start( self ):
        '''start the plugin'''
        self.start_dbus()
        
        self.user_connect_id = self.connect('user-online', self.user_connect)
        self.user_offline_id = self.connect('user-offline', self.user_offline)
        self.self_nick_changed_id = self.connect('self-nick-changed',
                                                  self.self_nick_changed)
        self.psm_changed_id = self.connect('personal-message-changed',
                                            self.psm_changed)

        self.enabled = True
        
    def stop( self ):    
        '''stop the plugin'''
        self.disconnect( self.user_connect_id )
        self.disconnect( self.user_offline_id )
        self.disconnect( self.self_nick_changed_id )
        self.disconnect( self.psm_changed_id )

        self.destroy_dbus_session()

        self.enabled = False
        
    def action( self ):
        pass
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        if dbusError != '':
            self.destroy_dbus_session()
            return (False, 'Can\'t Load dbus', dbusError)
            
        return (True, 'Ok')

    def start_dbus(self):
        '''Start dbus session'''
        self.destroy_dbus_session()
        self.session_bus = dbus.SessionBus()
        self.bus_name = dbus.service.BusName(BUS_NAME, bus=self.session_bus)
        self.dbus_object = EmeseneObject(self.bus_name, OBJECT_PATH, self)

    def destroy_dbus_session(self):
        '''Destroy current dbus session'''
        if self.dbus_object:
            try:
                dbus.service.Object.remove_from_connection(self.dbus_object)
            except AttributeError:
                pass
            self.dbus_object = None


    def user_connect(self, msnp, user, oldStatus):
        if(oldStatus=='FLN'):
            self.dbus_object.user_connect(user)

    def user_offline(self, msnp, user):
        self.dbus_object.user_offline(user)

    def self_nick_changed(self, msnp, old, nick):
        self.dbus_object.self_changed_nick(nick)

    def psm_changed(self, msnp, email, pm):
        self.dbus_object.psm_changed(email, pm)

try:
    class EmeseneObject(dbus.service.Object):
        def __init__(self, bus_name, object_path, main):
            dbus.service.Object.__init__(self, bus_name, object_path)
            self.main = main
            self.msn = main.msn
            self.controller = main.controller
            self.contact_manager = self.controller.contacts

        #Methods
        @dbus.service.method(BUS_NAME)
        def set_psm(self, text):
            self.contact_manager.set_message(text)
            return 'Psm Changed'

        @dbus.service.method(BUS_NAME)
        def set_media(self, text):
            self.contact_manager.set_media(text)
            return 'Current Media Changed'

        @dbus.service.method(BUS_NAME)
        def set_status(self, status):
            self.contact_manager.set_status(status)
            return 'Status Changed'

        @dbus.service.method(BUS_NAME)
        def set_nick(self, nick):
            self.contact_manager.set_nick(nick)
            return 'Nick changed'

        @dbus.service.method(BUS_NAME)
        def set_avatar(self, path):
            ''' set avatar '''
            if path != None and os.path.exists(path):
                self.controller.changeAvatar(path)
                return True
            return False


        @dbus.service.method(BUS_NAME)
        def open_conversation(self, email=''):
            try:
                self.controller.newConversation(None, email)
                return 'opened ' + str(email)
            except Exception, e:
                return 'error trying to start a conversation: ' + str(e)

        @dbus.service.method(BUS_NAME, out_signature='aa{ss}')
        def get_online_users(self):
            users = self.contact_manager.get_online_list()
            contacts = []
            for user in users:
                user_status = self.contact_manager.get_status(user.account)
                contact = {'email': user.account, 'name': user.nick, 'status': user_status}
                contacts.append(contact)
            return contacts

        @dbus.service.method(BUS_NAME)
        def get_contact_nick(self, email):
            return self.contact_manager.get_nick(email)
            
        @dbus.service.method(BUS_NAME)
        def get_psm(self, email):
            return self.contact_manager.get_message(email)


        #Signals
        @dbus.service.signal(BUS_NAME)
        def user_connect(self, user):
            pass

        @dbus.service.signal(BUS_NAME)
        def user_offline(self, email):
            pass

        @dbus.service.signal(BUS_NAME)
        def self_changed_nick(self, nick):
            pass

        @dbus.service.signal(BUS_NAME)
        def psm_changed(self, email, pm):
            pass
except Exception, e:
    dbusError = e



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Eval plugin is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Eval plugin is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Eval plugin v0.4
THIS IS A DEVELOPERS TOOL, USE AT YOUR OWN RISK!
Simple usage:
 /eval out("somestring")
 /eval out(dir())
 /eval out(slashAction.conversation.switchboard)
Remote usage:
 From the other side..:
   !eval out("somestring")
   !eval out(dir())
   !eval out(conversation.switchboard)
"""

VERSION = '0.4.2'

import sys
import time
import gobject
import traceback

import Plugin

class MainClass(Plugin.Plugin):
    '''Main plugin class'''
    
    def __init__(self, controller, msn):
        '''Constructor'''
        
        Plugin.Plugin.__init__(self, controller, msn)
        
        self.description = _('Evaluate python commands - USE AT YOUR OWN RISK')
        self.authors = {'Dx' : 'dx@dxzone.com.ar'}
        self.website = 'http://www.dxzone.com.ar'
        self.displayName = 'Eval'
        self.name = 'Eval'
        self.catchOutput = False
        self.controller = controller
        self.Slash = controller.Slash
        self.config = controller.config
        self.config.readPluginConfig(self.name)
        users = self.config.getPluginValue(self.name, 'users', '')
        self.allowed = users.split()
               
    def start(self):
        '''start the plugin'''
        self.Slash.register('eval', self.slashCommand, _('Run python commands'))
        conv_manager = self.controller.conversationManager
        self.receiveId = conv_manager.connect('receive-message', self.receive)
        self.enabled = True
        
    def slashCommand(self, slashAction):
        # variables init
        _ = None
        conversation = slashAction.conversation
        controller = self.controller
        msn = controller.msn
        params = str(slashAction.getParams())

        def out(text):
            slashAction.outputText(str(text))

        def send(text, includeCommand=False):
            text = str(text)
            if includeCommand:
                slashAction.outputText(' /eval ' + params + '\n' + \
                    str(text), True)
            else:
                slashAction.outputText(str(text), True)

        # keep old stdout
        if self.catchOutput:
            oldstdout = sys.stdout
        
        try:
            # catch output
            if self.catchOutput:
                sys.stdout = SlashOut(slashAction)
            # run the commands
            eval(compile(params, "<eval>", "exec"))
        except:
            exception = sys.exc_info()
            slashAction.outputText(traceback.format_exception(*exception)[-1])
            traceback.print_exception(*exception)
        
        # restore stdout
        if self.catchOutput:
            sys.stdout = oldstdout

    def receive(self, cm, conversation, mail, nick, message, format, charset):
    	'''Eval commands if mail is in allowed list'''
        if mail not in self.allowed or not message.startswith('!eval'):
            return
        cm.emit_stop_by_name('receive-message')
        message = message.replace('\r\n', '\n')
        _ = None
        controller = self.controller
        msn = controller.msn
        params = str(message.split('!eval ')[1] )

        def out(text):
            conversation.sendMessage(str(text))

        # keep old stdout
        if self.catchOutput:
            oldstdout = sys.stdout
        
        try:
            # catch output
            if self.catchOutput:
                sys.stdout = RemoteOut(conversation)
            # run the commands
            eval(compile(params, "<eval>", "exec"))
        except:
            exception = sys.exc_info()
            conversation.sendMessage(traceback.format_exception(*exception)[-1])
            traceback.print_exception(*exception)
        
        # restore stdout
        if self.catchOutput:
            sys.stdout = oldstdout

    def stop(self):
        '''stop the plugin'''
        self.Slash.unregister('eval')
        self.enabled = False
        
    def check(self):
        return (True, 'Ok')
        
    def configure(self):
    	'''Configuration Dialog'''
        l=[]
        l.append(Plugin.Option('users', str, _('Alowed users:'), '', 
                 self.config.getPluginValue( self.name, 'users', '' )))
        
        response = Plugin.ConfigWindow(_('Remote Configuration'), l).run()
        if response != None:
            self.users = str(response['users'].value)
            self.allowed = self.users.split()
            self.config.setPluginValue(self.name,'users', self.users)
        return True

# kept for historical purposes (not)
class FakeOut:
    def __init__(self):
        self.encoding = self.mode = self.name = self.newlines = None
        self.closed = False
        self.softspace = 0
    def close(self):
        pass
    def fileno(self):
        pass
    def flush(self):
        pass
    def isatty(self):
        pass
    def read(self):
        pass
    def tell(self):
        pass
    def write(self):
        pass
    def writelines(self):
        pass

class SlashOut(FakeOut):
    def __init__(self, slashAction):
        FakeOut.__init__(self)
        self.slashAction = slashAction
    def write(self,text):
        self.slashAction.outputText(text)

class RemoteOut(FakeOut):
    def __init__(self, slashAction):
        FakeOut.__init__(self)
        self.conversation = conversation
    def write(self,text):
        self.conversation.sendMessage(text)



#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#    This plugin uses some code of http://dev.jmoiron.net/projects/browser/xchat/mp3.py?rev=27&format=txt
#

VERSION = '0.3'

import os
import struct
import socket
if os.name == 'posix':
    import pwd

import CurrentSong

ERROR = ''

#Commands for sockets
CMD_GET_VERSION = 0         #
CMD_PLAYLIST_ADD = 1        #
CMD_PLAY = 2                #
CMD_PAUSE = 3               #
CMD_STOP = 4                #
CMD_IS_PLAYING = 5          #
CMD_IS_PAUSED = 6           #
CMD_GET_PLAYLIST_POS = 7    #
CMD_SET_PLAYLIST_POS = 8    #
CMD_GET_PLAYLIST_LENGTH = 9 #
CMD_PLAYLIST_CLEAR = 10     #
CMD_GET_OUTPUT_TIME = 11    #
CMD_JUMP_TO_TIME = 12       #
CMD_GET_VOLUME = 13         #
CMD_SET_VOLUME = 14         #
CMD_GET_SKIN = 15           #
CMD_SET_SKIN = 16
CMD_GET_PLAYLIST_FILE = 17  #
CMD_GET_PLAYLIST_TITLE = 18 #
CMD_GET_PLAYLIST_TIME = 19  #
CMD_GET_INFO = 20           #
CMD_GET_EQ_DATA = 21 # these two aren't implemented in
CMD_SET_EQ_DATA = 22 # xmms 1.0.1, and that's all I have
CMD_PL_WIN_TOGGLE = 23      #
CMD_EQ_WIN_TOGGLE = 24      #
CMD_SHOW_PREFS_BOX = 25     #
CMD_TOGGLE_AOT = 26         #
CMD_SHOW_ABOUT_BOX = 27     # doesn't seem to work :-(
CMD_EJECT = 28              #
CMD_PLAYLIST_PREV = 29      #
CMD_PLAYLIST_NEXT = 30      #
CMD_PING = 31
CMD_GET_BALANCE = 32        #
CMD_TOGGLE_REPEAT = 33      #
CMD_TOGGLE_SHUFFLE = 34     #
CMD_MAIN_WIN_TOGGLE = 35    #
CMD_PLAYLIST_ADD_URL_STRING = 36
CMD_IS_EQ_WIN = 37          #
CMD_IS_PL_WIN = 38          #
CMD_IS_MAIN_WIN = 39        #

class ClientPacketHeader:
    def __init__(self):
        self.version,self.cmd,self.length = 0,0,0
    def __repr__(self):
        return "<< %s : version: %s cmd: %s length: %s >>"\
            %(self.__class__.__name__,self.version,self.cmd,self.length)
    def encode(self):
        return struct.pack("=hhl",self.version,self.cmd,self.length)

class XmmsSocket:
    def __init__(self,session=0):
        self.status = True
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        if os.path.exists( "/tmp/xmms_%s.%d"%(pwd.getpwuid(os.geteuid())[0],session) ):
            sockFile = "/tmp/xmms_%s.%d"%(pwd.getpwuid(os.geteuid())[0],session)
        #elif os.path.exists( "/tmp/audacious_%s.%d"%(pwd.getpwuid(os.geteuid())[0],session) ):
        #    sockFile = "/tmp/audacious_%s.%d"%(pwd.getpwuid(os.geteuid())[0],session)
        else:
            self.status = False
            return
        if sockFile == None: return
        try:
            self.sock.connect( sockFile )
        except:
            self.status = False
        
    def send(self,cmd,data=""):
        if type(data) == type(1):
            data = struct.pack("=l",data)
        packet = struct.pack("=hhl",1,cmd,len(data))+data
        self.sock.send(packet)
    def sendf(self,cmd,fmt,args):
        data = apply(struct.pack,(fmt,)+args)
        self.send(cmd,data)

    def read_header(self, sock):
        head = ClientPacketHeader()
        head.version, head.cmd, head.length = \
            struct.unpack("=hhl",sock.recv(struct.calcsize( "=hhl" )))
            #struct.unpack("hhl",sock.recv(8))
        return head

    def get_reply(self):
        self.reply_header = self.read_header(self.sock)
        return self.sock.recv(self.reply_header.length)\
            .decode('utf-8', 'replace').encode('utf-8')

    def get_replyf(self,fmt):
        return struct.unpack(fmt,self.get_reply())
    def read_ack(self):
        self.get_reply()
        
    def isXmmsOpen(self):
        return self.status

class XmmsCommands:
    def __init__(self):
        pass
        
    def send_requestf(self, cmd, fmt, data=""):
        xmmsSocket = XmmsSocket()
        if xmmsSocket.isXmmsOpen():
            xmmsSocket.send(cmd, data)
            return xmmsSocket.get_replyf(fmt)
        return False
        
    def send_command(self, cmd, data=""):
        xmmsSocket = XmmsSocket()
        if xmmsSocket.isXmmsOpen():
            xmmsSocket.send(cmd,data)
            #xmmsSocket.read_ack() 
        return False

    def send_request_args(self, cmd, fmt, *args):
        xmmsSocket = XmmsSocket()
        if xmmsSocket.isXmmsOpen():
            xmmsSocket.sendf(cmd, fmt, args)
            return xmmsSocket.get_reply()
        return False
        
    def playlist_pos(self):
        return self.send_requestf(CMD_GET_PLAYLIST_POS,'=l')[0]
        
    def playlist_file(self , pos):
        return self.send_request_args(CMD_GET_PLAYLIST_FILE,'=l',pos)[:-1]
        
    def playlist_title(self, pos):
        return self.send_request_args(CMD_GET_PLAYLIST_TITLE,'=l',pos)[:-1]
        
    def playlist_time(self, pos):
        return self.send_requestf_args(CMD_GET_PLAYLIST_TIME,'=l','=l',pos)[0]
        
    def socket_info(self):
        """returns (bitrate, freq, nch)"""
        return self.send_requestf(CMD_GET_INFO, '=lll')

    def socket_playlist_length(self):
        return self.send_requestf(CMD_GET_PLAYLIST_LENGTH,'=l')[0]

    # in milliseconds
    def socket_output_time(self):
        return self.send_requestf(CMD_GET_OUTPUT_TIME,'=l')[0]

    def currentSong(self):
        return self.playlist_title(self.playlist_pos())

    def nextSong( self ):
        self.send_command(CMD_PLAYLIST_NEXT)
    
    def isPlaying(self):
        if XmmsSocket().isXmmsOpen():
            return self.send_requestf(CMD_IS_PLAYING,'=l')[0]
                
        return False


class Xmms(CurrentSong.CurrentSong):

    def __init__( self ):
        CurrentSong.CurrentSong.__init__( self )
        self.currentSong = ''
        self.xmms = XmmsCommands()
        
    def getSongPlain( self ):
        if not self.isPlaying():
            return ''
        return self.xmms.currentSong()
        
    def getCurrentSong( self ):
        if not self.isPlaying():
            return ''
        song = '\\0Music\\01\\0' + self.xmms.currentSong() + '\\0\\0'
        return song
            
    def nextSong( self ):
        self.xmms.nextSong()
            
    def isPlaying( self ):
        if self.xmms.isPlaying():
            return True
            
        return False
        
    def isRunning( self ):
        if self.xmms.isPlaying():
            return True
            
        return False

    def getStatus( self ):
        if os.name != 'posix':
            return ( False,  'This plugin only works in posix systems' )

        if ERROR:
            return ( False, ERROR )
        
        return ( True, 'Ok' )
        
    def check( self ):
        if self.currentSong != self.getCurrentSong():
            self.currentSong = self.getCurrentSong()
            return True
                
        return False



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''this module includes all MSNSLP related stuff'''

import random

class SLPMessage(object):
    '''Represents a MSNSLP/1.0 message
    No session initiation here, it's just a higher level interface
    to build/parse SLP messages'''

    TEMPLATE = '%s\r\n' \
        'To: <msnmsgr:%s>\r\n' \
        'From: <msnmsgr:%s>\r\n' \
        'Via: MSNSLP/1.0/TLP ;branch={%s}\r\n' \
        'CSeq: %s \r\n' \
        'Call-ID: {%s}\r\n' \
        'Max-Forwards: %s\r\n' \
        'Content-Type: %s\r\n' \
        'Content-Length: %s\r\n' \
        '\r\n' \
        '%s'
    
    INVITE_METHOD = 'INVITE MSNMSGR:%s MSNSLP/1.0'
    BYE_METHOD = 'BYE MSNMSGR:%s MSNSLP/1.0'
    ACK_METHOD = 'ACK MSNMSGR:%s MSNSLP/1.0'
    OK_STATUS = 'MSNSLP/1.0 200 OK'
    DECLINE_STATUS = 'MSNSLP/1.0 603 Decline'
    
    CONTENT_SESSION_REQ = 'application/x-msnmsgr-sessionreqbody'
    CONTENT_SESSION_CLOSE = 'application/x-msnmsgr-sessionclosebody'
    CONTENT_TRANS_RESP = 'application/x-msnmsgr-transrespbody'
    CONTENT_TRANS_REQ = 'application/x-msnmsgr-transreqbody'
    
    EUFGUID_DISPLAY = '{A4268EEC-FEC5-49E5-95C3-F126696BDBF6}'
    EUFGUID_FILE = '{5D3E02AB-6190-11D3-BBBB-00C04F795683}'
    #EUFGUID_WEBCAM = '{4BD96FC0-AB17-4425-A14A-439185962DC8}'
    #EUFGUID_GAMES = '{6A13AF9C-5308-4F35-923A-67E8DDA40C2F}' 
    
    def __init__(self, message=None, session=None):
        '''initializes most attributes to empty'''
        self.method = ''
        self.to_ = ''
        self.from_ = ''
        self.via = ''
        self.cseq = '0'
        self.call_id = ''
        self.max_forwards = '0'
        self.content_type = ''
    
        self.body = {}

        if session:
            self.to_ = session.to_
            self.from_ = session.from_
            self.via = session.via
            self.call_id = session.call_id
        elif message:
            self.parse(message)

    def parse(self, message):
        '''parses a message and sets the attributes to rebuild it
        the param must be without binary headers
        SLPErrors should me handled replying 500 internal server error'''
        try:
            header, body = message.split('\r\n\r\n', 1)
        except ValueError:
            raise SLPError("The message/body separator is missing")
            
        header = header.split("\r\n")
        body = body.split("\r\n")

        # body is easy, ok?
        self.body = to_dict(body)

        try:
            self.method = header.pop(0)
        except IndexError:
            raise SLPError("SLP Message header is empty "
                "(or starts with \\r\\n\\r\\n)")

        headerparams = to_dict(header)

        try:
            self.to_ = between(headerparams['To'], "<msnmsgr:", ">")
            self.from_ = between(headerparams['From'], "<msnmsgr:", ">")
            self.via = between(headerparams['Via'], ";branch={", "}")
            self.cseq = headerparams['CSeq'].strip()
            self.call_id = between(headerparams['Call-ID'], "{", "}")
            self.max_forwards = headerparams['Max-Forwards'].strip()
            self.content_type = headerparams['Content-Type'].strip()
        except KeyError:
            raise SLPError("Some of the required MSNSLP fields are missing")
        
    def __str__(self):
        '''return the message as a string'''
        
        # i take context apart to have it at the end
        context = None
        if 'Context' in self.body:
            context = self.body.pop('Context')  # take it out

        items = self.body.items()
        items.sort()

        if context:  # add it back 
            items.append(('Context', context))
            self.body['Context'] = context

        lines = []
        for key, value in items:
            lines.append(str(key) + ': ' + str(value))
        body = '\r\n'.join(lines) + '\r\n\r\n\0'
        
        return SLPMessage.TEMPLATE % (self.method, self.to_, self.from_,
            self.via, self.cseq, self.call_id, self.max_forwards,
            self.content_type, len(body), body)


# raised on SLP message parse error
class SLPError(Exception):
    '''A simple, named, easy to catch exception for all SLP parsing errors'''
    pass


### helper functions

def random_id():  #XXX
    '''return a random id according to msnp2p rules'''
    
    guid = []
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ))
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ) + "-")
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ) + "-")
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ) + "-")
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ) + "-")
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ))
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ))
    guid.append('%04X' % random.randint( 0x1111 , 0xAAFF ))
    
    return ''.join(guid)

def contains(string, substring):
    '''return True if string contains substring'''
    return (string.find(substring) != -1)

def between(string, start, end):
    '''Returns the string between start and end
    I wouldn't recommend using this outside this module...'''
    try:
        return string.split(start)[1].split(end)[0]
    except IndexError:
        raise SLPError("Couldn't parse field " + repr(string))

def to_dict(lines):
    '''Splits the list "lines" by ": " and returns a dict'''
    parameters = {}
    for line in lines:
        try:
            key, value = line.split(': ')
            parameters[key] = value
        except ValueError:
            continue
    return parameters

### messages
# this stuff could be moved to classmethods,
# but i think we need py2.3 somewhere
# also, the old classmethod syntax sucks

def invite(session, context, eufguid=SLPMessage.EUFGUID_DISPLAY):
    message = SLPMessage(session=session)
    message.method = SLPMessage.INVITE_METHOD % message.to_
    message.content_type = SLPMessage.CONTENT_SESSION_REQ
    message.body['EUF-GUID'] = eufguid
    message.body['SessionID'] = session.session_id
    message.body['AppID'] = session.app_id
    message.body['Context'] = context
    return message

def ok(session):
    message = SLPMessage(session=session)
    message.method = SLPMessage.OK_STATUS
    message.content_type = SLPMessage.CONTENT_SESSION_REQ
    message.cseq = '1'
    message.body['SessionID'] = session.session_id
    return message

def decline(session):
    message = SLPMessage(session=session)
    message.method = SLPMessage.DECLINE_STATUS
    message.content_type = SLPMessage.CONTENT_SESSION_CLOSE
    message.cseq = '1'
    message.body['SessionID'] = session.session_id
    return message

def bye(session):
    message = SLPMessage(session=session)
    message.method = SLPMessage.BYE_METHOD % message.to_
    message.content_type = SLPMessage.CONTENT_SESSION_CLOSE
    return message



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''an abstract class to define menus, and their content'''

import Object

class Accel(object):
    '''an abstract class representing an accel (for example Ctrl+Q)'''

    def __init__(self, key, ctrl=True, alt=False):
        '''class constructor'''
        self.key = key
        self.ctrl = ctrl
        self.alt = alt

class Base(Object.Object):
    '''the base class for the other items'''

    def __init__(self):
        Object.Object.__init__(self)
        self._enabled = True
        
        # this signals are emitted when the
        # element is enabled or disabled, the
        # implementation should connect to this
        # signal and do the stuff to change the
        # state of the element
        self.signal_add("enabled", 0)
        self.signal_add("disabled", 0)

    def _set_enabled(self, value):
        '''set the value of enabled to "value"'''
        self._enabled = value
        if value:
            self.signal_emit("enabled")
        else:
            self.signal_emit("disabled")

    def _get_enabled(self):
        '''return the value of self.enabled'''
        return self._enabled

    enabled = property(fset=_set_enabled, fget=_get_enabled)

class Selectable(Base):
    '''a base class that contains a text and a image representing an
    optional image to show. image is a tuple where the first element is
    a constant identifying the second value, the constant are defined on this 
    class (TYPE_STOCK, TYPE_IMAGE_PATH etc.)
    This class can emit the selected signal'''

    TYPE_NONE = 0
    TYPE_STOCK = 1
    TYPE_IMAGE_PATH = 2

    def __init__(self, text, image_type=TYPE_NONE, image=None, accel=None):
        '''class constructor'''
        Base.__init__(self)
        
        self.text = text
        self.image_type = image_type
        self.image = image
        self.accel = accel

        self.signal_add('selected', 0)

    def is_image_path(self):
        '''return True if image content is TYPE_IMAGE_PATH'''
        return self.image_type == Selectable.TYPE_IMAGE_PATH

    def is_stock(self):
        '''return True if image content is TYPE_STOCK'''
        return self.image_type == Selectable.TYPE_STOCK

    def get_image_path(self):
        '''return the image_path value if it's an image_path or None'''
        if self.is_image_path():
            return self.image

        return None

    def get_stock(self):
        '''return the stock value if it's a stock or None'''
        if self.is_stock():
            return self.image

        return None

class Menu(list, Selectable):
    '''a class that represent a container that can contain items or other
    menus'''

    def __init__(self, text='', image=None):
        '''class constructor'''
        Selectable.__init__(self, text, image)

class Option(list, Base):
    '''a class that can represent an option that can be selected deselected
    a group of items in which only one can be selected at a time.
     You must append Items to this object'''

    def __init__(self, selected_index=0):
        '''class constructor'''
        Base.__init__(self)

        self.selected_index = selected_index

class Item(Selectable):
    '''a nice name to identify an element that represent a label of text and
    a stock image that can be selected and emit the selected signal'''

    def __init__(self, text, image_type=Selectable.TYPE_NONE, image=None,
        accel=None):
        '''class constructor'''
        Selectable.__init__(self, text, image_type, image, accel)

class Togglable(Base): # Togglable? WTF? :P
    '''a class that represent an item that has a label and can be toggled'''

    def __init__(self, text, toggled=False):
        '''class constructor'''
        Base.__init__(self)

        self.text = text
        self.toggled = toggled

        # the new value of the toggled attribute
        self.signal_add('toggled', 1)

class CheckBox(Togglable):
    '''a nice name to a Togglable that is represented as a checkbox'''

    def __init__(self, text, toggled=False):
        '''class constructor'''
        Togglable.__init__(self, text, toggled)

class Radio(Togglable):
    '''a nice name to a Togglable that is represented as a radio'''

    def __init__(self, text):
        '''class constructor'''
        Togglable.__init__(self, text, False)

def build(element):
    '''build an element with the defined toolkit and return it'''
    raise NotImplementedError

def build_menu_bar(menu):
    '''build a menu with the form of a menu bar'''
    raise NotImplementedError

def build_pop_up(menu, x=0, y=0):
    '''build a pop up menu, if positions are given and needed by the toolkit
    will be used to show the pop up on that position'''
    raise NotImplementedError





# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
import gobject

import emesenelib.common
from emesenelib.common import escape, unescape

# constants
TAGS_NONE = 0
TAGS_PANGO = 1
TAGS_HTML = 2


# "data types" parse text for an specific target

class DefaultDataType( object ):
    tags = TAGS_NONE

    def __init__( self, text, parser ):
        self.parser = parser
        self.text = text
        self.theme = parser.getTheme()
        
    def get( self, escaped=True ):
        data = ''
        for item in self.parser.parse( self, self.text ):
            if escaped:
                data += str(item)
            else:
                data += unescape(str(item))
        return data
    
    def parseAll( self ):
        data = ''
        for item in self.text:
            try:
                data += item.getHtml()
            except:
                data += str( item )
        return data
        
    def getDataType( self ):
        return 'default'

class PangoDataType( DefaultDataType ): 
    '''This returns a list with strings and pixbufs'''
    tags = TAGS_PANGO

    def __init__( self, text, parser ):
        DefaultDataType.__init__( self, text, parser )
        self.parser = parser
        self.text = text
        self.theme = parser.getTheme()
        self.regex = re.compile("<span([^>]*)>(.*)$")
        self.tagsopen = []

    def getDataType( self ):
        return 'pango'

    def get( self, smileys=True, cache={} ):
        '''returns list, not a string!'''
        ret = []
        for item in self.parser.parse( self, self.text ):
            if type(item) in (str, unicode, Url):
                ret = self.handystringhandler( ret, item )
                
            elif type(item) == Smiley:
                if not smileys:
                    ret = self.addstr(ret, str(item))
                    continue
                if item.smiley in cache:
                    item.pixbuf = cache[ item.smiley ]
                else:
                    item.pixbuf = item.getPixbuf(False)
                ret.append(item)
        return [self.fix(i) for i in ret]
        
    def addstr( self, ret, string ):
        '''adds a string to the last item if possible'''
        if len(ret) > 0 and type(ret[-1]) == str and ret[-1] != '\n':
            ret[-1] += str(string)
        else:
            ret.append(str(string))
        return ret

    def handystringhandler( self, ret, item ):
        '''separates a string by the \n if needed'''
        if type(item) == Url:
            item = str(item)
        if str(item).find('\n') != -1:
            olditem, item = str(item).split('\n', 1)
            ret = self.addstr(ret, olditem)
            ret.append('\n')
        ret = self.addstr(ret, item)
        return ret

    def fix(self, data):
        '''close tags before a smiley and reopen after it'''
        if type(data) == str:
            pre = ''.join(self.tagsopen)
            self.tagsopen = []
            data = self.regex.sub(self.fixhandler, pre+data)
            closedTags = data.count('</span>')
            if closedTags > 0:
                self.tagsopen = self.tagsopen[:-closedTags]
            data = data + ('</span>' * len(self.tagsopen))
        return data

    def fixhandler(self, data):
        '''self.regex.sub handler'''
        group0, group1 = data.groups()
        group1 = self.regex.sub(self.fixhandler, group1)
        self.tagsopen.append('<span%s>' % group0)
        return '<span%s>%s' % (group0,group1)


class UserListDataType( PangoDataType ): 
    '''This returns a list with strings and pixbufs'''
    tags = TAGS_PANGO

    def __init__( self, text, parser ):
        PangoDataType.__init__( self, text, parser )

    def getDataType( self ):
        return 'userlist'

    def get( self, smileys=True, cache={} ):
        '''returns list, not a string!'''
        ret = []

        # unpack the tuple :D
        template = self.text[0]
        params = self.text[1:]

        # pseudo format()
        template = template.split("%s")

        for part in params:
            ret = self.handystringhandler(ret, template.pop(0))
            ret = self.multipart_parse(ret, part, smileys, cache)

        if len(template) > 0:
            ret = self.handystringhandler(ret, template.pop(0))

        return [self.fix(i) for i in ret]
    
    ###
    # EVERYTHING (!) BELOW THIS COMMENT IS THE RESULT
    # OF REALLY HEAVY CODE REUTILISATION
    ###
    # (inside the class, i mean)

    def multipart_parse( self, ret, part, smileys, cache ):
        '''sends a chunk to the parser, adds to ret'''
        for item in self.parser.parse( self, part ):
            if type(item) in (str, unicode, Url):
                ret = self.handystringhandler( ret, item )
                
            elif type(item) == Smiley:
                if not smileys:
                    ret = self.addstr(ret, str(item))
                    continue
                if item.smiley in cache:
                    item.pixbuf = cache[ item.smiley ]
                else:
                    item.pixbuf = item.getPixbuf(False)
                ret.append(item)
        return ret

class ConversationDataType( DefaultDataType ):
    tags = TAGS_HTML

    def __init__( self, text, parser ):
        DefaultDataType.__init__( self, text, parser )
        self.conversation = None
        self.ink = None
        self.username = None

    def subOCE( self, data ):
        ce = data.groups()[0]
        msnOM = self.conversation.switchboard.msn.getMsnObjectsManager()
        filename = msnOM.getById(unescape(ce)).filename.replace("%", "%25")
        return '<img src="file://%s" alt="%s"/>' % (filename, ce)
            
    def subICE( self, data ):
        ce = data.groups()[0]
        customEmoticons = self.conversation.customEmoticons.get( self.username )
        return '<object type="application/x-emesene-emoticon" class="%s" ' \
            'data="%s"></object>' % (customEmoticons[unescape(ce)], ce)
        
    def get( self, urls=True, smileys=True ):
        data = ''
        cePattern=''
        if self.ink:
            return '<object type="application/x-emesene-ink" class="%s">' \
                '</object>' % (self.text,)

        if smileys == True and self.conversation != None:
            if self.username == self.conversation.switchboard.user:
                sub = self.subOCE
                msnOM = self.conversation.switchboard.msn.getMsnObjectsManager()
                for ce in msnOM.getIds():
                    cePattern += re.escape(escape(ce)) + '|'
            else:
                sub = self.subICE
                customEmoticons = self.conversation.customEmoticons\
                    .get( self.username )
                for ce in customEmoticons:
                    cePattern += re.escape(escape(ce)) + '|'
        
        text = self.text

        for item in self.parser.parse( self, text ):
            if type( item ) in (str, unicode):
                if cePattern!= '':
                    item = re.sub( '('+cePattern[:-1]+')', sub, item )
                data += item
            elif type(item) == Url:
                if urls:
                    data += item.getHtml()
                else:
                    data += str( item )
            elif type(item) == Smiley:
                if smileys:
                    data += item.getHtml()
                else:
                    data += str( item )
                    
        return data
        
    def setConversation( self, conversation ):
        self.conversation = conversation
        
    def setInk( self, value ):
        self.ink = True
        
    def setUser( self, username ):
        if username != None: self.username = username.lower()
            
    def getDataType( self ):
        return 'conversation'


# classes that describe objects inside the text

class Smiley(object):
    def __init__( self, smiley, theme ):
        self.smiley = smiley
        self.theme = theme
        self.pixbuf = None
        self.scaled = False
    
    def getHtml( self ):
        if self.theme.getSmileyPath( self.smiley ) == None:
            print 'Can\'t find the smiley %s'%self.smiley
            return escape( self.smiley )
        
        path = self.theme.getSmileyPath( self.smiley ).replace("%", "%25")
        if path.startswith('/'):    
            url = 'file://' + path
        else:
            url = 'file:///' + path # Windows
        return '<img src="%s" alt="%s"/>' % (url, escape(self.smiley))
        
    def getPixbuf( self, animated=True ):
        pixbuf = self.theme.getSmiley( self.smiley, animated )
        return pixbuf
        
    def __str__( self ):
        return escape(self.smiley)
        
    def __repr__(self):
        return '<smiley ' + self.smiley + '>'

    def scale( self, cache, size ):
        if self.scaled: return
        if self.smiley in cache:
            self.pixbuf = cache[self.smiley]
        else:
            if not self.pixbuf:
                self.pixbuf = self.getPixbuf(False)
            self.pixbuf = self.pixbuf.scale_simple(size, size, 2) #bilinear
            self.scaled = True
            cache[self.smiley] = self.pixbuf
        
class Url(object):
    def __init__( self, url ):
        self.url = url

    def getHtml( self ):
        href = self.url
        if self.url[:4] == 'www.':
            href  = 'http://'+self.url
        elif self.url[:4] == 'ftp.':
            href  = 'ftp://'+self.url
        return '<a href="%s">%s</a>' % (escape(href), escape(self.url))
    
    def getPango( self ):
        return emesenelib.common.escape( self.url )
        
    def __str__( self ):
        return emesenelib.common.escape( self.url )
        
    def __repr__( self ):
        return '<url ' + self.url + '>'

# the unified parser class itself

class UnifiedParser( gobject.GObject ):
    __gsignals__ = {
        #see the FilterData class docstring
        'filter' : (gobject.SIGNAL_RUN_LAST, str, [gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT] ),
        }
    
    custompatterns = {} # {name: (regex, handler)}
    
    def __init__( self, theme ):
        gobject.GObject.__init__( self )
        self.theme = theme
        self.urlsPattern = "(?P<url>(?<![A-Za-z0-9])((?:https?\:\/\/|ftp\:\/\/|www\.|ftp\.)(?:\S*[;:~]?)))"
        self.smileyPattern=''
        for smiley in theme.getSmileysList():
            self.smileyPattern += re.escape( smiley ) + '|'
        self.smileyPattern = '(?P<smiley>'+self.smileyPattern[:-1]+')'
        
        self.recompileRegex()
        
    def getTheme( self ):
        return self.theme
    
    def getParser( self, text, dataType=DefaultDataType ):
        return dataType( text, self )
    
    def parse( self, datatype, text ):
        list = []
        def handler( data ):
            group = data.groupdict()
            if group['smiley'] != None:
                list.append( Smiley(group['smiley'], self.theme) )
            elif group['url'] != None:
                list.append( Url( group['url'] ) )
            elif group['other'] != None:
                other = escape(str(group['other']))
                if len(list)>0 and type( list[-1] ) == str:
                    list[-1] += other
                else:
                    list.append(other)
        self.reALL.sub( handler, text )
        data = FilterData( list=list )
        self.emit( 'filter', datatype, data )
        return data.list
        
    def recompileRegex( self ):
        custom = ''
        for name in self.custompatterns.keys():
            self.custompatterns[name][0]
            custom += '(?P<' + name + '>' + self.custompatterns[name][0] + ')|'
        pattern = self.urlsPattern + '|' + \
                  self.smileyPattern + '|' + \
                  custom + '(?P<other>.)'
        self.reALL = re.compile( pattern,re.DOTALL )

# filters in gobject! yay!

class FilterData(object):
    '''A class to hold data that can be modified even in
    gobject signal handlers. That makes them "filters"'''

    def __init__( self, **kw ):
        for key in kw.keys():
            setattr(self, key, kw[key])

    def serialize( self, list ):
        '''Converts a list into a format string and a list of objects
        returns (str, list), where the second list is the object list'''

        format = ''
        objects = []

        for item in list:
            if type(item) in (str, unicode):
                # string, just escape and add it
                format += str(item).replace("%", "%%")
            else:
                # object, represented with a "%s" in the format string
                format += '%s'
                objects.append(item)
        return (format, objects)

    def deserialize( self, format, objects ):
        '''Converts a format string and a list of objects into a list
        returns list [str, obj, str, ...]'''

        list = []
        format = str(format)

        while len(format) > 0:
            # find the next object
            pos = format.find("%s")

            if pos == -1:
                #no more objects in string
                list.append(format.replace("%%","%"))
                return list

            if pos != 0 and format[pos-1] == "%":
                # escaped %, include it in text
                text, format = format[:pos+2], format[pos+2:]
                object = None
            else:
                # real object, exclude it from text
                text, format = format[:pos], format[pos+2:]
                object = objects.pop(0)
            
            # unescape % from deserialized string
            list.append(text.replace("%%","%"))
            if object != None:
                list.append(object)

        return list

if __name__=='__main__':
    import Theme
    
    theme = Theme.Theme('')
    parser = UnifiedParser( theme )
    #data = ':D :o :P http://someurl.com/~das/:Dx/index.od?asd=asd&asd [b]test[/b] :D <:o)(G)(@)")'
    data = '") <tag> [b][/b] %s % omg omg'
    print "Conversation:"
    print '  ', parser.getParser( data, ConversationDataType ).get()
    print "UserList:"
    print '  ', parser.getParser( ('%s',data), UserListDataType ).get()
    print "Default:"
    print '  ', parser.getParser( data ).get()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import CurrentSong
import sys
import gettext
import os
import socket
import urllib, urllib2


def _extract_fields(reply, fields):
    """Return a dict associating a value to each requested field from a MPD
    reply.

    Parameters:
        reply - MPD reply (string)
        fields - sequence of fields (lowercase) we'll associate a value to.
    
    Here's an example of an MPD reply:
    
        Artist: foo
        Title: bar
        Album: other
        OK
    """
    # Initialize dict values to empty strings in case they are not in
    # the MPD reply.
    ret = {}
    for field in fields:
        ret[field] = ""

    for line in reply.splitlines():
        if ":" in line:
            field, value = line.split(":", 1)
            field = field.strip().lower()
            if field in fields:
                ret[field] = value.strip()
    return ret

RETRYNUM = 1000

class Mpd( CurrentSong.CurrentSong ):
    
    customConfig = {
        'host': '127.0.0.1',
        'port': '6600',
	'password': '',
    }
    
    def __init__(self):
        CurrentSong.CurrentSong.__init__(self)
        self._socket = None
        self._retNum = 0
        
        self.song_info = {
            'artist': '',
            'title': '',
            'album': '',
        }

        dictCommandMPD = {
            'play': ( self.cmd_Play, 'Start playing',False ),
            'stop': ( self.cmd_Stop, 'Stop playing',False ),
            'pause': (self.cmd_Pause, 'Pause player',False ),
            'next': (self.cmd_Next, 'Jump to next song',False),
            'prev': (self.cmd_Prev, 'Jump to previous song',False),
            'getvol': (self.cmd_GetVolume, 'get volume',False),
            'setvol': (self.cmd_SetVolume, 'set volume <int vol>. The range of volume is 0-100',False)
        }

        self.dictCommand.update(dictCommandMPD)

    def updateConfig(self):
        if self.connected():
            self._socket.close()
            self._socket = None
        self._connect()
        for item in self.song_info.keys():
            self.song_info[item] = ""

    def tryReconnect():
        if self._retNum > RETRYNUM:
            self._retNum = 0
            self._connect()
        else:
            self._retNum += 1

    def _getSocket(self):
        if self._socket == None:
            self.tryReconnect()
        return self._socket

    def _connect(self):
        if self.connected():
            return
        self.log('info', 'trying to connect')
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self._socket.connect((self.customConfig.get('host', 
                '127.0.0.1'), int(self.customConfig.get('port', 6600))))
            if not self._socket.recv(128).startswith("OK"):
                raise
            if self.customConfig.get('password', ''):
                self.log('info', 'sending password')
                self._socket.send('password ' +  self.customConfig['password'] +'\n')
                if not self._socket.recv(128).startswith("OK"):
                    self.log('error', 'wrong password')
                    raise socket.error
        except socket.error, msg:
            self.log('error', 'can\'t connect to %s:%s' % \
                (self.customConfig.get('host', '127.0.0.1'), 
                    self.customConfig.get('port', 6600)))
            self.status = 'not running'
            self._socket.close()
            self._socket = None
        else:
            self.log('info', 'connected sucessfully')
            self.status = 'connected'
        return 
    
    def connected(self):
        if self._socket is None:
            return False

        return True
    
    def start(self):
        self._connect()
        return True

    def check(self):
        if self.checkSongChanged():
            self.artist = self.getArtist()
            self.title = self.getTitle()
            self.album = self.getAlbum()
            return True
        return False
        
    def stop(self):
        '''Close connection'''
        if self.connected():
            self._socket.close()
        self._socket = None

    def setHost(self, host):
        self.host = host

    def setPort(self, port):
        self.port = port

    def setPassword(self, password):
        self.password = password

    def getArtist(self):
        return self.song_info["artist"]
    
    def getAlbum(self):
        return self.song_info["album"]
    
    def getTitle(self):
        return self.song_info["title"]
    
    def sendCmd(self, cmd):
        try:
            sock = self._getSocket()
            sock.send("%s\n" % cmd)
            ret = sock.recv(1024)
            # Command failed
            if ret.startswith("ACK"):
                return None
            return ret
        except:
            return None
        
    def isPlaying(self):
        status = self.sendCmd("status")
        if status is not None:
            if _extract_fields(status, ["state"])["state"] == "play":
                return True

        return False

    def checkSongChanged(self):
        if not self.isPlaying():
            if "".join(self.song_info.values()) != "":
                # MPD stopped playing, so empty all fields
                for item in self.song_info.keys():
                    self.song_info[item] = ""      
                return True
        else: 
            song_rpl = self.sendCmd("currentsong")
            if song_rpl is not None:
                song_info = _extract_fields(song_rpl, self.song_info.keys())
                if self.song_info != song_info:
                    self.song_info = song_info
                    return True
            
        return False

    def cmd_Pause(self,args):
        ret = self.sendCmd('pause')
        if ret != None and ret.startswith('OK'):
            return ( True, 'Player Paused' )
        return ( False, 'Error Pausing Player' )

    def cmd_Play(self,args):
        ret = self.sendCmd('play')
        if ret != None and ret.startswith('OK'):
            return ( True, 'Player start playing' )
        return ( False, 'Error on playing' )

    def cmd_Stop(self,args):
        ret = self.sendCmd('stop')
        if ret != None and ret.startswith('OK'):
            return ( True, 'Player Stopped' )
        return ( False, 'Error on Stopping Player' )
    
    def cmd_Next(self,args):
        ret = self.sendCmd('next')
        if ret != None and ret.startswith('OK'):
            return ( True, 'Next Track' )
        return ( False, 'Error jumping to next track' )

    def cmd_Prev(self,args):
        ret = self.sendCmd('prev')
        if ret != None and ret.startswith('OK'):
            return ( True, 'Previous Track' )
        return ( False, 'Error jumping to previos track' )

    def cmd_GetVolume(self,args):
        ret = self.sendCmd('status')
        if ret != None:
            try:
                volume = _extract_fields(ret,['volume'])['volume']
                return ( True, 'Volume: ' + str(volume))
            except:
                pass
        return ( False, 'Error Getting Volume' )

    def cmd_SetVolume(self,args):
        try:
            vol = int(args)
            if vol < 0 or vol > 100:
                raise
        except:
            return ( False, 'setvol needs an integer value,range 0-100' )
        
        ret = self.sendCmd('setvol ' + str(vol))
        if ret != None and ret.startswith('OK'):
            return ( True, 'Volume setted to ' + str(vol) )
        return ( False, 'Error Setting Volume' )

        
